/*
 *  CPPGlobals.h
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/2/12.
 *  Copyright 2012 Dan Libisch. All rights reserved.
 *
 */

#ifndef _CPP_GLOBALS_H_
#define _CPP_GLOBALS_H_

#include <ApplicationServices/ApplicationServices.h>
#include <objc/objc.h> //allows c++ access to objective-c types
#include <string>

#include <vector>
using std::vector;

#include <OpenGL/gl.h>
#include <OpenGL/glu.h>

#include "CPPFileSelector.h"
#include "CPPMessage.h"
#include "CPPNode.h"

#include "Vector.h"

class CPPApplication;
class CPPMainWindow;
class CPPWindow;

// global pointer to application node
extern CPPApplication *CPPApp;
void SetAppPointer(CPPApplication *app);
CPPApplication* GetAppPointer(void);

// temporary object for testing
void drawCube(void);


// class for color
class CPPColor
{
public:
	CGFloat r;
	CGFloat g;
	CGFloat b;
	CGFloat a;
	
	CPPColor();
	~CPPColor();
	
	void SetColor(CGFloat R, CGFloat G, CGFloat B, CGFloat A=1.0);
	void SetColor(CPPColor clr);
};

// global menu colors
extern CPPColor labelNormalColor;
extern CPPColor labelHiliteColor;
extern CPPColor labelDisabledColor;
extern CPPColor menuActiveItemColor;
extern CPPColor menuBackgroundColor;

CGRect GetPositionScreenFrame(CGPoint point);

id GetNSColor(CPPColor color);
id GetNSColor(CGFloat r, CGFloat g, CGFloat b, CGFloat a);

int CPPRunAlertPanel(const char *title, const char *msg, const char *btn1, const char *btn2="", const char *btn3="");

// timer functions
double CPPGetTimer(void);
void StartTimer(double &mTimer);
double GetElapsedTime(double &mTimer);


// window style mask
enum {
    CPPBorderlessWindowMask				= 0,
    CPPTitledWindowMask					= 1 << 0,
    CPPClosableWindowMask				= 1 << 1,
    CPPMiniaturizableWindowMask			= 1 << 2,
    CPPResizableWindowMask				= 1 << 3,

	// Specifies a window with textured background (eg. metal)
    CPPTexturedBackgroundWindowMask		= 1 << 8,
	
	// Specifies a window that ignores the userSpaceScaleFactor of the NSScreen on which it is created.
	// Currently restricted to borderless windows (NSBorderlessWindowMask)
    CPPUnscaledWindowMask				= 1 << 11,

	// Specifies a window whose titlebar and toolbar have a unified look - that is, a continuous background
    CPPUnifiedTitleAndToolbarWindowMask	= 1 << 12
};

// Panel specific styleMask
enum {
	CPPUtilityWindowMask		= 1 << 4,
	CPPDocModalWindowMask		= 1 << 6,
	CPPNonactivatingPanelMask	= 1 << 7,
	CPPHUDWindowMask			= 1 << 13
};


// structure for OpenGL margin offsets
struct ViewMargins
{
	int			left;	// left margin
	int			top;	// top margin
	int			right;	// right margin
	int			bottom;	// bottom margin
	
	ViewMargins(void)
	{
		left = 0;
		top = 0;
		right = 0;
		bottom = 0;
	}
};


// event modifier keys
enum
{
	cppModifierNone = 0,
	cppAlphaShiftKey,
	cppShiftKey,
	cppControlKey,
	cppAlternateKey,
	cppCommandKey,
	cppCommandAlternateKey,
	cppCommandControlKey,
	cppCommandShiftKey,
	cppAlternateControlKey,
	cppAlternateShiftKey,
	cppControlShiftKey
};
bool ModifierKey(int modKey, CGEventFlags flags);


// UI fonts
enum
{
	cppSystemFont = 0,
	cppBoldSystemFont,
	cppLabelFont,
	cppTitleBarFont,
	cppMenuFont,
	cppMenuBarFont,
	cppMessageFont,
	cppPaletteFont,
	cppToolTipsFont,
	cppControlContentFont
};

enum
{
    cppLeftTextAlignment = 0,		// Visually left aligned
    cppRightTextAlignment,			// Visually right aligned
    cppCenterTextAlignment,			// Visually centered
    cppJustifiedTextAlignment,		// Fully-justified. The last line in a paragraph is natural-aligned.
    cppNaturalTextAlignment			// Indicates the default alignment for script
};


CGFloat SystemFontSize(void);
CGFloat SmallSystemFontSize(void);
CGFloat LabelFontSize(void);

id GetNSString(const char *str);
id GetNSFont(int font, CGFloat size);

CGSize GetStringDrawingSize(int font, CGFloat size, const char *str, float padding);

// event mouse states
int GetMouseDeltaX(CGEventRef cgEvent);
int GetMouseDeltaY(CGEventRef cgEvent);
int GetMouseDeltaW(CGEventRef cgEvent);
int GetMouseClickCount(CGEventRef cgEvent);

CGPoint GetLocationInFrame(CGPoint pt, CGRect frame);

// even key codes
int GetEventKeyCode(CGEventRef cgEvent);
int GetCharKeyCode(const char *c);
const char* ModifierKeyToCharacter(int mKey);

// Virtual keycodes according to the key position on an ANSI-standard US keyboard.
// Copied from Carbon/HIToolbox/Events.h simply to avoid including Carbon framework.
enum {
	CPP_KEY_A                    = 0x00,
	CPP_KEY_S                    = 0x01,
	CPP_KEY_D                    = 0x02,
	CPP_KEY_F                    = 0x03,
	CPP_KEY_H                    = 0x04,
	CPP_KEY_G                    = 0x05,
	CPP_KEY_Z                    = 0x06,
	CPP_KEY_X                    = 0x07,
	CPP_KEY_C                    = 0x08,
	CPP_KEY_V                    = 0x09,
	CPP_KEY_B                    = 0x0B,
	CPP_KEY_Q                    = 0x0C,
	CPP_KEY_W                    = 0x0D,
	CPP_KEY_E                    = 0x0E,
	CPP_KEY_R                    = 0x0F,
	CPP_KEY_Y                    = 0x10,
	CPP_KEY_T                    = 0x11,
	CPP_KEY_1                    = 0x12,
	CPP_KEY_2                    = 0x13,
	CPP_KEY_3                    = 0x14,
	CPP_KEY_4                    = 0x15,
	CPP_KEY_6                    = 0x16,
	CPP_KEY_5                    = 0x17,
	CPP_KEY_Equal                = 0x18,
	CPP_KEY_9                    = 0x19,
	CPP_KEY_7                    = 0x1A,
	CPP_KEY_Minus                = 0x1B,
	CPP_KEY_8                    = 0x1C,
	CPP_KEY_0                    = 0x1D,
	CPP_KEY_RightBracket         = 0x1E,
	CPP_KEY_O                    = 0x1F,
	CPP_KEY_U                    = 0x20,
	CPP_KEY_LeftBracket          = 0x21,
	CPP_KEY_I                    = 0x22,
	CPP_KEY_P                    = 0x23,
	CPP_KEY_L                    = 0x25,
	CPP_KEY_J                    = 0x26,
	CPP_KEY_Quote                = 0x27,
	CPP_KEY_K                    = 0x28,
	CPP_KEY_Semicolon            = 0x29,
	CPP_KEY_Backslash            = 0x2A,
	CPP_KEY_Comma                = 0x2B,
	CPP_KEY_Slash                = 0x2C,
	CPP_KEY_N                    = 0x2D,
	CPP_KEY_M                    = 0x2E,
	CPP_KEY_Period               = 0x2F,
	CPP_KEY_Grave                = 0x32,
	CPP_KEY_KeypadDecimal        = 0x41,
	CPP_KEY_KeypadMultiply       = 0x43,
	CPP_KEY_KeypadPlus           = 0x45,
	CPP_KEY_KeypadClear          = 0x47,
	CPP_KEY_KeypadDivide         = 0x4B,
	CPP_KEY_KeypadEnter          = 0x4C,
	CPP_KEY_KeypadMinus          = 0x4E,
	CPP_KEY_KeypadEquals         = 0x51,
	CPP_KEY_Keypad0              = 0x52,
	CPP_KEY_Keypad1              = 0x53,
	CPP_KEY_Keypad2              = 0x54,
	CPP_KEY_Keypad3              = 0x55,
	CPP_KEY_Keypad4              = 0x56,
	CPP_KEY_Keypad5              = 0x57,
	CPP_KEY_Keypad6              = 0x58,
	CPP_KEY_Keypad7              = 0x59,
	CPP_KEY_Keypad8              = 0x5B,
	CPP_KEY_Keypad9              = 0x5C
};

// keycodes for keys that are independent of keyboard layout
enum {
	CPP_KEY_Return                    = 0x24,
	CPP_KEY_Tab                       = 0x30,
	CPP_KEY_Space                     = 0x31,
	CPP_KEY_Delete                    = 0x33,
	CPP_KEY_Escape                    = 0x35,
	CPP_KEY_Command                   = 0x37,
	CPP_KEY_Shift                     = 0x38,
	CPP_KEY_CapsLock                  = 0x39,
	CPP_KEY_Option                    = 0x3A,
	CPP_KEY_Control                   = 0x3B,
	CPP_KEY_RightShift                = 0x3C,
	CPP_KEY_RightOption               = 0x3D,
	CPP_KEY_RightControl              = 0x3E,
	CPP_KEY_Function                  = 0x3F,
	CPP_KEY_F17                       = 0x40,
	CPP_KEY_VolumeUp                  = 0x48,
	CPP_KEY_VolumeDown                = 0x49,
	CPP_KEY_Mute                      = 0x4A,
	CPP_KEY_F18                       = 0x4F,
	CPP_KEY_F19                       = 0x50,
	CPP_KEY_F20                       = 0x5A,
	CPP_KEY_F5                        = 0x60,
	CPP_KEY_F6                        = 0x61,
	CPP_KEY_F7                        = 0x62,
	CPP_KEY_F3                        = 0x63,
	CPP_KEY_F8                        = 0x64,
	CPP_KEY_F9                        = 0x65,
	CPP_KEY_F11                       = 0x67,
	CPP_KEY_F13                       = 0x69,
	CPP_KEY_F16                       = 0x6A,
	CPP_KEY_F14                       = 0x6B,
	CPP_KEY_F10                       = 0x6D,
	CPP_KEY_F12                       = 0x6F,
	CPP_KEY_F15                       = 0x71,
	CPP_KEY_Help                      = 0x72,
	CPP_KEY_Home                      = 0x73,
	CPP_KEY_PageUp                    = 0x74,
	CPP_KEY_ForwardDelete             = 0x75,
	CPP_KEY_F4                        = 0x76,
	CPP_KEY_End                       = 0x77,
	CPP_KEY_F2                        = 0x78,
	CPP_KEY_PageDown                  = 0x79,
	CPP_KEY_F1                        = 0x7A,
	CPP_KEY_LeftArrow                 = 0x7B,
	CPP_KEY_RightArrow                = 0x7C,
	CPP_KEY_DownArrow                 = 0x7D,
	CPP_KEY_UpArrow                   = 0x7E
};

#endif