/*
 *  CPPMenu.h
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/27/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */

#ifndef _CPP_MENU_H_
#define _CPP_MENU_H_


#include "CPPControl.h"
#include "CPPPanel.h"


// forward declaration for menu item class
class CPPMenuItem;

// window menu bar class
class CPPMenuBar : public CPPControl
{
protected:	
	CPPColor		topColor;
	CPPColor		bottomColor;
	
	CGFloat			barHeight;
	CGFloat			lastOffset;
	
	bool			mDragging;
	bool			shouldClose;
	
	double			dragTimer;
	
	
public:
	// debugging
	//const char *name;
	
	CPPMenuBar();
	virtual ~CPPMenuBar();
	
	void SetBackgroundColors(CPPColor t, CPPColor b);
	CPPColor GetTopColor(void);
	CPPColor GetBottomColor(void);
	
	virtual void Draw(void);
	virtual bool Message(int msg, void *data=NULL);
	
	void SetIsDragging(bool flag);
	bool IsDragging(void);
	
	CPPMenuItem* AddMenuItem(const char *str);
	void AddOffset(CGFloat off);
	CGFloat GetOffset(void);
	
	// events 
	virtual bool HandleEvents(CGEventRef cgEvent);
	
	virtual void LeftMouseDown(CGEventRef cgEvent);
	virtual void LeftMouseDragged(CGEventRef cgEvent);
	virtual void LeftMouseUp(CGEventRef cgEvent);
	
	virtual void MouseMoved(CGEventRef cgEvent);
	virtual void ScrollWheel(CGEventRef cgEvent);
	
	virtual void KeyDown(CGEventRef cgEvent);
	virtual void KeyUp(CGEventRef cgEvent);
	
	
	// objective-c call functions
	virtual id InitWithFrame(CGRect cRect);
	
	void SetCppNsMenuBarPointer(CPPMenuBar *mb);
	
};


// menu class
class CPPMenu : public CPPPanel
{
protected:
	CGFloat			lastOffset;
	bool				active;
	//bool			mDragging;
	
public:
	// debugging
	//const char *name;
	
	CPPMenu();
	virtual ~CPPMenu();
	
	virtual void Draw(void);
	virtual bool Message(int msg, void *data=NULL);
	
	virtual void LeftMouseDown(CGEventRef cgEvent);
	virtual void LeftMouseDragged(CGEventRef cgEvent);
	virtual void LeftMouseUp(CGEventRef cgEvent);
	
	virtual void MouseMoved(CGEventRef cgEvent);
	virtual void ScrollWheel(CGEventRef cgEvent);
	
	virtual void KeyDown(CGEventRef cgEvent);
	virtual void KeyUp(CGEventRef cgEvent);
	
	void SetIsActive(bool flag);
	bool IsActive(void);
	
	//void SetIsDragging(bool flag);
	//bool IsDragging(void);
	
	CPPMenuItem* AddMenuItem(const char *str, CPPNode *trg=NULL, int msg=0, void *data=NULL, const char *htKy="", int mdKy=0);
	void AddOffset(CGFloat off);
	CGFloat GetOffset(void);
	
	void OpenMenuPanel(CGPoint origin);
	
};


// menu item class
class CPPMenuItem : public CPPControl
{
protected:
	CPPControlLabel		*stateLabel;
	CPPControlLabel		*itemLabel;
	CPPControlLabel		*hotkeyLabel;
	
	CPPMenu				*subMenu;
	
	CGFloat				offset;
	
	bool				menuBarItem;
	bool				isEnabled;
	
	int					menuState;
	
	CPPNode				*target;
	int					message;
	void				*msgData;
	
	int					hotKey;
	int					modKey;
	
public:
	
	CPPMenuItem();
	virtual ~CPPMenuItem();
	
	virtual void Draw(void);
	virtual bool Message(int msg, void *data=NULL);
	
	void SetOffset(CGFloat off);
	CGFloat GetOffset(void);
	
	void SetTarget(CPPNode *trg);
	CPPNode* GetTarget(void);
	
	void SetMessage(int msg);
	int GetMessage(void);
	
	void SetMessageData(void *mDat);
	void* GetMessageData(void);
	
	void SetHotKey(int hk);
	int GetHotKey(void);
	
	void SetModifierKey(int mk);
	int GetModifierKey(void);
	
	void SetIsMenuBarItem(bool flag);
	bool IsMenuBarItem(void);
	
	void InitMenuStateLabel(void);
	void InitMenuHotKeyLabel(void);
	
	void SetIsEnabled(bool flag);
	bool IsEnabled(void);
	
	void SetMenuState(int state);
	int GetMenuState(void);
		
	CPPMenu* AddSubMenu(void);
	CPPMenu* GetSubMenu(void);
	
	void BlinkMenuSelection(void);
	const char* GetName();
	
	// events
	virtual void LeftMouseDown(CGEventRef cgEvent);
	virtual void LeftMouseUp(CGEventRef cgEvent);
	
	virtual void KeyDown(CGEventRef cgEvent);
	
	// objective-c call functions
	virtual id InitWithFrame(CGRect cRect);
	id InitWithFrameAndLabel(CGRect cRect, const char *str, const char *hKey="", const char *mKey=" ");
	
	void SetCppNsMenuItemPointer(CPPMenuItem *mi);
	
};


#endif