/*
 *  CPPNode.h
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/19/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */

#ifndef _CPP_NODE_H_
#define _CPP_NODE_H_

#include <ApplicationServices/ApplicationServices.h>
//#include <objc/objc.h> //allows c++ access to objective-c types

#include "CPPMessage.h"

#include <vector>
using std::vector;

// forward declaration for node list class
class CPPNodeList; 


// base class for all CPP classes
class CPPNode
{
protected:
	CPPNode			*parent;
	CPPNodeList		*children;

	// messages
	CPPNode			*nodeTrg;
	CPPMessage		nodeMsg;

public:
	CPPNode();
	virtual ~CPPNode();
	
	virtual void Draw(void);
	virtual bool Message(int msg, void *data=NULL);
	
	void SendMessage(CPPNode *n, int m, void *d=NULL);
	void ClearMessage(void);
	bool ValidMessage(void);
	
	void SetParent(CPPNode *pr);
	CPPNode* GetParent(void);
	
	CPPNodeList* GetChildren(void);
	
	CPPNode* GetChild(void);
	CPPNode* GetPrev(void);
	CPPNode* GetNext(void);
	CPPNode* GetLastChild(void);
	
	void Remove(void);
	void Insert(CPPNode *n);
	
};

// list class for nodes
class CPPNodeList
{
protected:
	vector<CPPNode*> ndList;
	vector<CPPNode*>::iterator ndIter;
	
public:
	CPPNodeList()
	{
	}
	~CPPNodeList()
	{
		Clear();
	}

	void Append(CPPNode *n)
	{
		ndList.push_back(n);
	}
	
	void SetIteratorIndex(int index)
	{
		ndIter = ndList.begin();
		
		if(index < ndList.size())
		{
			int i = 0;
			while(i < index)
			{
				i++;
				ndIter++;
			}
		}
	}
		
	void Insert(int index, CPPNode *n)
	{
		if(index < ndList.size())
		{
			SetIteratorIndex(index);
			ndList.insert(ndIter,n);
		}
	}
	
	void Remove(CPPNode *n)
	{
		int index = GetIndex(n);
		if(index < 0) return;
		
		SetIteratorIndex(index);
		ndList.erase(ndIter);
	}
	
	void Clear(void)
	{
		ndList.clear();
	}
	
	int GetCount(void)
	{
		return ndList.size();
	}
	
	int GetIndex(CPPNode *n)
	{
		int index = -1;
		
		for(int i=0; i<ndList.size(); i++) 
		{
			if(ndList[i] == n) index = i;
		}
		
		return index;
	}
	
	CPPNode* GetNode(int index)
	{
		if(index < ndList.size())
		{
			return ndList[index];
		}
		
		return NULL;
	}
	
};

#endif