/*
 *  CPPApplication.cpp
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 2/27/12.
 *  Copyright 2012 Dan Libisch. All rights reserved.
 *
 */

#include "CPPApplication.h"
#include "CPPMainWindow.h"



/*****************************************************************************/
// CPPApplication C++ class functions

CPPApplication::CPPApplication()
{
	InitSharedApplication();
	
    mainWindow = NULL;
	wndCloseApp = TRUE;
}

CPPApplication::~CPPApplication()
{
	DeleteMainWindow();
}

void CPPApplication::DeleteMainWindow(void)
{
	if(mainWindow)
	{
		delete mainWindow;
		mainWindow = NULL;
	}
}

CPPMainWindow* CPPApplication::GetMainWindow(void)
{
	return mainWindow;
}

void CPPApplication::SetIsRunning(bool flag)
{
	isRunning = flag;
}

void CPPApplication::SetMainWindowClosesApplication(bool flag)
{
	wndCloseApp = flag;
}

void CPPApplication::SetMainAppColors(void)
{
	labelNormalColor.SetColor(0.0f,0.0f,0.0f,1.0f);
	labelHiliteColor.SetColor(1.0f,1.0f,1.0f,1.0f);
	labelDisabledColor.SetColor(0.5f,0.5f,0.5f,1.0f);
	menuActiveItemColor.SetColor(0.2f,0.3f,1.0f,1.0f);
	menuBackgroundColor.SetColor(1.0f,1.0f,1.0f,1.0f);
}

void CPPApplication::InitMainWindow(int w, int h)
{
	// create main window
	mainWindow = new CPPMainWindow();
	mainWindow->SetParent(this);
	
	CGRect cRect = CGRectMake(0,0,w,h);
	
	unsigned int style = CPPClosableWindowMask | CPPMiniaturizableWindowMask | CPPResizableWindowMask;
	mainWindow->InitWithContentRect(cRect, style);
	
	CGFloat minWidth = 100;
	CGFloat minHeight = 44;
	
	// add window menu bar
	CGFloat mBarHeight = 16;
	CPPMenuBar *mBar = mainWindow->InitWindowMenuBar(cRect,mBarHeight);
	if(mBar)
	{
		CPPMenuItem *mItem1 = mBar->AddMenuItem("Menu1");
		if(mItem1)
		{
			CPPMenu *subMn1 = mItem1->AddSubMenu();
			if(subMn1)
			{
				CPPMenuItem *sbm1Item1 = subMn1->AddMenuItem("Menu1 Item1 Longer");
				//if(!sbm1Item1) printf("Failed to add sbm1Item1\n");
				CPPMenuItem *sbm1Item2 = subMn1->AddMenuItem("Menu1 Item2");
				//if(!sbm1Item2) printf("Failed to add sbm1Item2\n");
				CPPMenuItem *sbm1Item3 = subMn1->AddMenuItem("Menu1 Item3");
				//if(!sbm1Item3) printf("Failed to add sbm1Item3\n");
			}
			//else printf("Failed to add subMn1\n");
		}

		CPPMenuItem *mItem2 = mBar->AddMenuItem("Menu2");
		if(mItem2)
		{
			CPPMenu *subMn2 = mItem2->AddSubMenu();
			if(subMn2)
			{
				CPPMenuItem *sbm2Item1 = subMn2->AddMenuItem("Menu2 Item1");
				//if(!sbm2Item1) printf("Failed to add sbm2Item1\n");
				CPPMenuItem *sbm2Item2 = subMn2->AddMenuItem("Menu2 Item2 Longer");
				//if(!sbm2Item2) printf("Failed to add sbm2Item2\n");
				CPPMenuItem *sbm2Item3 = subMn2->AddMenuItem("Menu2 Item3");
				//if(!sbm2Item3) printf("Failed to add sbm2Item3\n");
				
				CPPMenu *subMn3 = sbm2Item3->AddSubMenu();
				if(subMn3)
				{
					CPPMenuItem *sbm3Item1 = subMn3->AddMenuItem("Menu3 Item1");
					//if(!sbm3Item1) printf("Failed to add sbm3Item1\n");
					CPPMenuItem *sbm3Item2 = subMn3->AddMenuItem("Menu3 Item2");
					//if(!sbm3Item2) printf("Failed to add sbm3Item2\n");
					CPPMenuItem *sbm3Item3 = subMn3->AddMenuItem("Exit Application",this,cppTerminateApplication,mBar->GetNSObjectPointer(),"Q",cppCommandKey);
					//if(!sbm3Item3) printf("Failed to add sbm3Item3\n");
				}
				//else printf("Failed to add subMn3\n");
			}
			//else printf("Failed to add subMn2\n");
		}
	}
	
	// add OpenGL view
	if(mainWindow->CreateOpenGL3DViewPort(cRect))
	{
		mainWindow->GetOpenGLView()->SetViewMargins(0,mBarHeight,0,0);
		ViewMargins vMargins = mainWindow->GetOpenGLView()->GetViewMargins();
		minWidth += vMargins.left+vMargins.right;
		minHeight += vMargins.top+vMargins.bottom;
	}
	
	mainWindow->SetMinSize(minWidth,minHeight);
	mainWindow->SetTitle(GetApplicationName());
	
	mainWindow->SetDelegate(mainWindow->GetNSObjectPointer());
    mainWindow->SetAcceptsMouseMovedEvents(true);
	
	mainWindow->MakeFirstResponder(mainWindow->GetNSObjectPointer());
    mainWindow->MakeKeyAndOrderFront();
	
	mainWindow->Message(cppWindowDidResize);// called to do an initial refresh
	
}

void CPPApplication::RunLoop(void)
{
	SetIsRunning(TRUE);
	
	do
	{
		CGEventRef cgEvent = PollNextEvent();
		if(cgEvent)
		{
			if(DispatchCGEvents(cgEvent))
			{
				if(mainWindow) mainWindow->Draw();
				ForwardEventToNSApplication();
				
				if(nodeTrg && ValidMessage())
				{
					nodeTrg->Message(nodeMsg.msg, nodeMsg.data);
					ClearMessage();
				}
			}
		}
		
	}while(isRunning);
}

int CPPApplication::Execute(void)
{
	InitMainMenu();
	InitMainWindow(832,624);
	//SetMainWindowClosesApplication(FALSE);
	FinishLaunching();
	SetAppPointer(this);
	SetMainAppColors();
	
	RunLoop();
	
	return 0;
}

bool CPPApplication::MainWindowClosesApplication(void)
{
	return wndCloseApp;
}

bool CPPApplication::ApplicationShouldTerminate(void)
{
	if(!mainWindow->WindowIsOpen() && MainWindowClosesApplication()) return TRUE;
	
	if(!CPPRunAlertPanel("CPPAppKit","Are you sure you want to quit","OK","Cancel",""))
	{
		mainWindow->MakeKeyAndOrderFront();
		return FALSE;
	}
	else return TRUE;
}

bool CPPApplication::ApplicationWillTerminate(void)
{
	return TRUE;
}

bool CPPApplication::Message(int msg, void *data)
{
	switch(msg)
	{
		case cppGenericMessage:
		{
			//printf("CPPApplication::Message = cppGenericMessage \n");
			return TRUE;
		}
		case cppTerminateApplication:
		{
			//printf("CPPApplication::Message = cppTerminateApplication \n");
			id sndr = (id)data;
			if(sndr) TerminateApplication(sndr);
			return TRUE;
		}
		case cppApplicationWillTerminate:
		{
			//printf("CPPApplication::Message = cppApplicationWillTerminate \n");
			return ApplicationWillTerminate();
		}
		case cppApplicationShouldTerminate:
		{
			//printf("CPPApplication::Message = cppApplicationShouldTerminate \n");
			return ApplicationShouldTerminate();
		}
		case cppMainWindowClosesApplication:
		{
			//printf("CPPApplication::Message = cppMainWindowClosesApplication \n");
			return MainWindowClosesApplication();
		}
	}
	
	return TRUE;
}

// application events
void CPPApplication::LeftMouseDown(CGEventRef cgEvent)// for debugging
{
	//CGPoint mPos = CGEventGetLocation(cgEvent);
	//printf("screenPoint.x = %f, screenPoint.y = %f\n",mPos.x, mPos.y);
}

void CPPApplication::KeyDown(CGEventRef cgEvent)
{
	//printf("KCPPApplication::KeyDown \n");
}

void CPPApplication::KeyUp(CGEventRef cgEvent)
{
	//printf("CPPApplication::KeyUp \n");
}

void CPPApplication::FlagsChanged(CGEventRef cgEvent)
{
	//printf("CPPApplication::FlagsChanged \n");
	
}
