/*
 *  CPPGlobals.cpp
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/3/12.
 *  Copyright 2012 Dan Libisch. All rights reserved.
 *
 */

#include "CPPGlobals.h"

#include "CPPApplication.h"

// global application pointer
CPPApplication *CPPApp;

void SetAppPointer(CPPApplication *app)
{
	CPPApp = app;
}

CPPApplication* GetAppPointer(void)
{
	return CPPApp;
}


// global menu colors
CPPColor labelNormalColor;
CPPColor labelHiliteColor;
CPPColor labelDisabledColor;
CPPColor menuActiveItemColor;
CPPColor menuBackgroundColor;

// CPPColor functions
CPPColor::CPPColor()
{
	r = 0.0;
	g = 0.0;
	b = 0.0;
	a = 1.0;
}

CPPColor::~CPPColor()
{
}

void CPPColor::SetColor(CGFloat R, CGFloat G, CGFloat B, CGFloat A)
{
	r = R;
	g = G;
	b = B;
	a = A;
}

void CPPColor::SetColor(CPPColor clr)
{
	r = clr.r;
	g = clr.g;
	b = clr.b;
	a = clr.a;
	
}


// timer functions
void StartTimer(double &mTimer)
{
	mTimer = CPPGetTimer();
	//printf("Timer started\n");
}

double GetElapsedTime(double &mTimer)
{
	double time = CPPGetTimer() - mTimer;
	//printf("Elapsed time = %f\n",time);
	return 	time;
}


// modifier keys
bool ModifierKey(int modKey, CGEventFlags flags)
{
	switch(modKey)
	{
		case cppModifierNone:
			return TRUE;
		case cppAlphaShiftKey:
			return flags & kCGEventFlagMaskAlphaShift;
		case cppShiftKey:
			return flags & kCGEventFlagMaskShift;
		case cppControlKey:
			return flags & kCGEventFlagMaskControl;
		case cppAlternateKey:
			return flags & kCGEventFlagMaskAlternate;
		case cppCommandKey:
			return flags & kCGEventFlagMaskCommand;
		case cppCommandAlternateKey:
			return flags & kCGEventFlagMaskCommand && flags & kCGEventFlagMaskAlternate;
		case cppCommandControlKey:
			return flags & kCGEventFlagMaskCommand && flags & kCGEventFlagMaskControl;
		case cppCommandShiftKey:
			return flags & kCGEventFlagMaskCommand && flags & kCGEventFlagMaskShift;
		case cppAlternateControlKey:
			return flags & kCGEventFlagMaskAlternate && flags & kCGEventFlagMaskControl;
		case cppAlternateShiftKey:
			return flags & kCGEventFlagMaskAlternate && flags & kCGEventFlagMaskShift;
		case cppControlShiftKey:
			return flags & kCGEventFlagMaskControl && flags & kCGEventFlagMaskShift;
	}
	
	return FALSE;
}


// event mouse state
int GetMouseDeltaX(CGEventRef cgEvent)
{
	return CGEventGetIntegerValueField(cgEvent, kCGMouseEventDeltaX);
}

int GetMouseDeltaY(CGEventRef cgEvent)
{
	return CGEventGetIntegerValueField(cgEvent, kCGMouseEventDeltaY);
}

int GetMouseDeltaW(CGEventRef cgEvent)
{
	return CGEventGetIntegerValueField(cgEvent, kCGScrollWheelEventDeltaAxis1);
}

int GetMouseClickCount(CGEventRef cgEvent)
{
	return CGEventGetIntegerValueField(cgEvent, kCGMouseEventClickState);
}

CGPoint GetLocationInFrame(CGPoint pt, CGRect frame)
{
	CGPoint location;
	
	location.x = pt.x - frame.origin.x;
	location.y = pt.y - frame.origin.y;
	
	return location;
}


// event key code
int GetEventKeyCode(CGEventRef cgEvent)
{
	return CGEventGetIntegerValueField(cgEvent, kCGKeyboardEventKeycode);
}

int GetCharKeyCode(const char *c)
{
	// alphabet keys
	if(c == "A" || c == "a") return CPP_KEY_A;
	else if(c == "B" || c == "b") return CPP_KEY_B;
	else if(c == "C" || c == "c") return CPP_KEY_C;
	else if(c == "D" || c == "d") return CPP_KEY_D;
	else if(c == "E" || c == "e") return CPP_KEY_E;
	else if(c == "F" || c == "f") return CPP_KEY_F;
	else if(c == "G" || c == "g") return CPP_KEY_G;
	else if(c == "H" || c == "h") return CPP_KEY_H;
	else if(c == "I" || c == "i") return CPP_KEY_I;
	else if(c == "J" || c == "j") return CPP_KEY_J;
	else if(c == "K" || c == "k") return CPP_KEY_K;
	else if(c == "L" || c == "l") return CPP_KEY_L;
	else if(c == "M" || c == "m") return CPP_KEY_M;
	else if(c == "N" || c == "n") return CPP_KEY_N;
	else if(c == "O" || c == "o") return CPP_KEY_O;
	else if(c == "P" || c == "p") return CPP_KEY_P;
	else if(c == "Q" || c == "q") return CPP_KEY_Q;
	else if(c == "R" || c == "r") return CPP_KEY_R;
	else if(c == "S" || c == "s") return CPP_KEY_S;
	else if(c == "T" || c == "t") return CPP_KEY_T;
	else if(c == "U" || c == "u") return CPP_KEY_U;
	else if(c == "V" || c == "v") return CPP_KEY_V;
	else if(c == "W" || c == "w") return CPP_KEY_W;
	else if(c == "X" || c == "x") return CPP_KEY_X;
	else if(c == "Y" || c == "y") return CPP_KEY_Y;
	else if(c == "Z" || c == "z") return CPP_KEY_Z;
	
	//  top row numbers
	else if(c == "0") return CPP_KEY_0;
	else if(c == "1") return CPP_KEY_1;
	else if(c == "2") return CPP_KEY_2;
	else if(c == "3") return CPP_KEY_3;
	else if(c == "4") return CPP_KEY_4;
	else if(c == "5") return CPP_KEY_5;
	else if(c == "6") return CPP_KEY_6;
	else if(c == "7") return CPP_KEY_7;
	else if(c == "8") return CPP_KEY_8;
	else if(c == "9") return CPP_KEY_9;
	
	// symbol keys
	else if(c == "-") return CPP_KEY_Minus;
	else if(c == "=") return CPP_KEY_Equal;
	else if(c == "[") return CPP_KEY_LeftBracket;
	else if(c == "]") return CPP_KEY_RightBracket;
	else if(c == ";") return CPP_KEY_Semicolon;
	else if(c == "'") return CPP_KEY_Quote;
	else if(c == ",") return CPP_KEY_Comma;
	else if(c == ".") return CPP_KEY_Period;
	else if(c == "/") return CPP_KEY_Slash;
	
	
	return -1;
}

const char* ModifierKeyToCharacter(int mKey)
{
	switch(mKey)//⌃⇧⌥⌘
	{
		case cppModifierNone:
			return " ";
		case cppShiftKey:
			return "⇧";
		case cppControlKey:
			return "⌃";
		case cppAlternateKey:
			return "⌥";
		case cppCommandKey:
			return "⌘";
		case cppCommandAlternateKey:
			return "⌥⌘";
		case cppCommandControlKey:
			return "⌃⌘";
		case cppCommandShiftKey:
			return "⇧⌘";
		case cppAlternateControlKey:
			return "⌃⌥";
		case cppAlternateShiftKey:
			return "⇧⌥";
		case cppControlShiftKey:
			return "⌃⇧";
			
	}
	
	return "";
}


// temporary object for testing
void drawCube(void)
{	
	// draw cube
	glBegin(GL_QUADS);
	
	// top face
	glColor3f(0.0f,0.8f,0.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f( 1.0f, 1.0f, 1.0f);
	
	// bottom face
	glColor3f(0.8f,0.8f,0.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f,-1.0f);
	
	// front face
	glColor3f(0.0f,0.0f,0.8f);
	glVertex3f( 1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);
	
	// back face
	glColor3f(0.0f,0.8f,0.8f);
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f(-1.0f,-1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);
	
	// left face
	glColor3f(0.8f,0.0f,0.8f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f,-1.0f,-1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);
	
	// right face
	glColor3f(0.8f,0.0f,0.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f, 1.0f);
	
	glEnd();
}
