/*
 *  CPPMenu.cpp
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/25/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */


#include "CPPApplication.h"
#include "CPPMenu.h"


/*****************************************************************************/
// CPPMenuBar C++ class functions

CPPMenuBar::CPPMenuBar()
{
	topColor.SetColor(1.0f,1.0f,1.0f,1.0f);
	bottomColor.SetColor(0.7f,0.7f,0.7f,1.0f);
	lastOffset = 10.0f;
	SetViewMargins(0,0,-1,-1);
	shouldClose = FALSE;
}

CPPMenuBar::~CPPMenuBar()
{
}

void CPPMenuBar::SetBackgroundColors(CPPColor t, CPPColor b)
{
	topColor = t;
	bottomColor = b;
}

CPPColor CPPMenuBar::GetTopColor(void)
{
	return topColor;
}

CPPColor CPPMenuBar::GetBottomColor(void)
{
	return bottomColor;
}

void CPPMenuBar::SetIsDragging(bool flag)
{
	mDragging = flag;
}

bool CPPMenuBar::IsDragging(void)
{
	return mDragging;
}

CPPMenuItem* CPPMenuBar::AddMenuItem(const char *str)
{
	CPPMenuItem *mItem = new CPPMenuItem(); if(!mItem) return NULL;
	
	mItem->SetIsMenuBarItem(TRUE);
	id menuItem = mItem->InitWithFrameAndLabel(GetViewRect(),str);
	AddSubview(mItem);

	CGPoint origin = CGPointMake(0.0f,0.0f);
	origin.x += GetOffset();
	AddOffset(mItem->GetViewSize().width);
	//mItem->SetFrameOrigin(origin);
	mItem->SetViewOrigin(origin);
	
	CGRect miRect = mItem->GetViewRect();
	miRect.origin.x = origin.x;
	mItem->SetViewRect(miRect);
	mItem->Message(cppResizeContent,&miRect);
	
	return mItem;
}

void CPPMenuBar::AddOffset(CGFloat off)
{
	lastOffset += off;
}

CGFloat CPPMenuBar::GetOffset(void)
{
	return lastOffset;
}

void CPPMenuBar::Draw(void)
{
}

bool CPPMenuBar::Message(int msg, void *data)
{
	switch(msg)
	{
		case cppResizeContent:
		{
			CGRect *cntRect = (CGRect*)data;
			if(cntRect)
			{
				ResizeViewRect(*cntRect);
				for(int i=0; i < children->GetCount(); i++)
					children->GetNode(i)->Message(cppResizeContent,&vwRect);
			}
			break;
		}
		case cppControlActivate:
		{
			SetIsActive(TRUE);
			break;
		}
		case cppControlDeactivate:
		{
			SetIsActive(FALSE);
			break;
		}
	}
	
	return TRUE;
}

bool CPPMenuBar::HandleEvents(CGEventRef cgEvent)
{
	CGEventType type = CGEventGetType(cgEvent);
	
	mxDelta = GetMouseDeltaX(cgEvent);
	myDelta = GetMouseDeltaY(cgEvent);
	mwDelta = GetMouseDeltaW(cgEvent);
	mClick = GetMouseClickCount(cgEvent);
	
	switch(type)
	{
		case kCGEventLeftMouseDown:
			LeftMouseDown(cgEvent);
			break;
		case kCGEventLeftMouseUp:
			LeftMouseUp(cgEvent);
			break;
		case kCGEventMouseMoved:
			MouseMoved(cgEvent);
			break;
		case kCGEventScrollWheel:
			ScrollWheel(cgEvent);
			break;
		case kCGEventLeftMouseDragged:
			LeftMouseDragged(cgEvent);
			break;
		case kCGEventKeyDown:
			KeyDown(cgEvent);
			break;
		case kCGEventKeyUp:
			KeyUp(cgEvent);
			break;
	}
	
	if(IsActive())
	{
		if(shouldClose)
		{
			Message(cppControlDeactivate);
			shouldClose = FALSE;
		}
		return FALSE;
	}
		
	return TRUE;
}

void CPPMenuBar::LeftMouseDown(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	if(!IsActive())
	{	
		//printf("screenPoint.x = %f, screenPoint.y = %f\n",mPos.x, mPos.y);
		if(MouseInViewRect(mPos))
		{
			StartTimer(dragTimer);
			SetIsDragging(TRUE);
			for(int i=0; i < children->GetCount(); i++)
			{
				CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(children->GetNode(i));
				if(menuItem)
				{
					if(menuItem->MouseInViewRect(mPos))
					{
						menuItem->Message(cppControlActivate);
						SetIsActive(TRUE);
						SetNeedsDisplay(TRUE);
					}
					else
						if(menuItem->IsActive()) menuItem->Message(cppControlDeactivate);
				}
			}
		}
		//else printf("mouse not in menu bar\n");
	}
	else
	{
		shouldClose = TRUE;
	}
}

void CPPMenuBar::LeftMouseDragged(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	if(IsActive() && MouseInViewRect(mPos))
	{	
		for(int i=0; i < children->GetCount(); i++)
		{
			CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(children->GetNode(i));
			if(menuItem)
			{
				if(menuItem->MouseInViewRect(mPos))
					menuItem->Message(cppControlActivate);
				else
					if(menuItem->IsActive()) menuItem->Message(cppControlDeactivate);
			}
		}
		SetNeedsDisplay(TRUE);
	}
}

void CPPMenuBar::LeftMouseUp(CGEventRef cgEvent)
{
	if(IsActive())
	{
		if(GetElapsedTime(dragTimer) < 0.4 && IsDragging()) SetIsDragging(FALSE);
		
		if(IsDragging())
		{
			for(int i=0; i < children->GetCount(); i++)
			{
				CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(children->GetNode(i));
				if(menuItem && menuItem->IsActive()) menuItem->Message(cppControlDeactivate);
			}
			SetNeedsDisplay(TRUE);
			SetIsDragging(FALSE);
			SetIsActive(FALSE);
		}
	}
	else
	{
		for(int i=0; i < children->GetCount(); i++)
		{
			CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(children->GetNode(i));
			if(menuItem && menuItem->IsActive()) menuItem->Message(cppControlDeactivate);
		}
		SetNeedsDisplay(TRUE);
	}
}

void CPPMenuBar::MouseMoved(CGEventRef cgEvent)
{
	if(IsActive())
	{
		CGPoint mPos = CGEventGetLocation(cgEvent);
		if(MouseInViewRect(mPos))
		{
			for(int i=0; i < children->GetCount(); i++)
			{
				CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(children->GetNode(i));
				if(menuItem)
				{
					if(menuItem->MouseInViewRect(mPos))
						menuItem->Message(cppControlActivate);
					else
						if(menuItem->IsActive()) menuItem->Message(cppControlDeactivate);
				}
			}
			SetNeedsDisplay(TRUE);
		}
	}
}

void CPPMenuBar::ScrollWheel(CGEventRef cgEvent)
{
	
}

void CPPMenuBar::KeyDown(CGEventRef cgEvent)
{
	
}

void CPPMenuBar::KeyUp(CGEventRef cgEvent)
{
	
}



/*****************************************************************************/
// CPPMenu C++ class functions

CPPMenu::CPPMenu()
{
	lastOffset = 4.0;
}

CPPMenu::~CPPMenu()
{
}

void CPPMenu::Draw(void)
{
}

void CPPMenu::SetIsActive(bool flag)
{
	active = flag;
}

bool CPPMenu::IsActive(void)
{
	return active;
}


bool CPPMenu::Message(int msg, void *data)
{
	switch(msg)
	{
		case cppControlActivate:
		{
			SetIsActive(TRUE);
			break;
		}
		case cppControlDeactivate:
		{
			SetIsActive(FALSE);
			CPPView *mView = GetContentView();
			if(mView)
			{
				CPPNodeList *vwChildren = mView->GetChildren();
				if(vwChildren)
				{
					for(int i=0; i < vwChildren->GetCount(); i++)
					{
						CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(vwChildren->GetNode(i));
						if(menuItem)
						{
							if(menuItem->IsActive()) menuItem->Message(cppControlDeactivate);
						}
					}
				}
			}
			ClosePanel();
			break;
		}
		case cppWindowDidResize:
		{
			//printf("CPPMenu::Message = cppWindowDidResize \n");
			return WindowDidResize();
		}
	}
	
	return TRUE;
}

void CPPMenu::LeftMouseDown(CGEventRef cgEvent)
{
	if(WindowIsOpen())
	{
		CPPView *mView = GetContentView();
		if(mView)
		{
			CPPNodeList *vwChildren = mView->GetChildren();
			if(vwChildren)
			{
				CGPoint mPos = CGEventGetLocation(cgEvent);
				if(!mView->MouseInViewRect(mPos))
				{
					CPPMenuItem *prItem = dynamic_cast<CPPMenuItem*>(parent);
					if(prItem && !prItem->MouseInViewRect(mPos))
					{
						bool inSubMenu = FALSE;
						
						for(int i=0; i < vwChildren->GetCount(); i++)
						{
							CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(vwChildren->GetNode(i));
							if(menuItem)
							{
								if(menuItem->IsActive())
								{
									CPPMenu *subMenu = menuItem->GetSubMenu();
									if(subMenu)
									{
										CPPView *subView = subMenu->GetContentView();
										if(subView->MouseInViewRect(mPos)) inSubMenu = TRUE;
									}
									else menuItem->Message(cppControlDeactivate);
								}
							}
						}
						if(!inSubMenu)
						{
							Message(cppControlDeactivate);
							prItem->Message(cppControlDeactivate);
							prItem->SetNeedsDisplay(TRUE);
						}
					}
				}
			}
		}
	}
}

void CPPMenu::LeftMouseDragged(CGEventRef cgEvent)
{
	MouseMoved(cgEvent);
}

void CPPMenu::LeftMouseUp(CGEventRef cgEvent)
{
	if(WindowIsOpen())
	{
		CPPView *mView = GetContentView();
		if(mView)
		{
			CPPNodeList *vwChildren = mView->GetChildren();
			if(vwChildren)
			{
				CGPoint mPos = CGEventGetLocation(cgEvent);
				if(mView->MouseInViewRect(mPos))
				{
					for(int i=0; i < vwChildren->GetCount(); i++)
					{
						CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(vwChildren->GetNode(i));
						if(menuItem && menuItem->IsActive())
						{
							menuItem->Message(cppControlDeactivate);
						}
					}
					mView->SetNeedsDisplay(TRUE);
					CPPMenuItem *prItem = dynamic_cast<CPPMenuItem*>(parent);
					if(prItem)
					{
						prItem->Message(cppControlDeactivate);
						prItem->SetNeedsDisplay(TRUE);
					}
				}
				else
				{
					CPPMenuItem *prItem = dynamic_cast<CPPMenuItem*>(parent);
					if(prItem && !prItem->IsMenuBarItem())
					{
						prItem->Message(cppControlDeactivate);
						prItem->SetNeedsDisplay(TRUE);
					}
				}
			}
		}
	}
}

void CPPMenu::MouseMoved(CGEventRef cgEvent)
{
	if(WindowIsOpen())
	{
		CPPView *mView = GetContentView();
		if(mView)
		{
			CPPNodeList *vwChildren = mView->GetChildren();
			if(vwChildren)
			{
				CGPoint mPos = CGEventGetLocation(cgEvent);
				if(mView->MouseInViewRect(mPos))
				{
					for(int i=0; i < vwChildren->GetCount(); i++)
					{
						CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(vwChildren->GetNode(i));
						if(menuItem)
						{
							if(menuItem->MouseInViewRect(mPos))
								menuItem->Message(cppControlActivate);
							else
								if(menuItem->IsActive()) menuItem->Message(cppControlDeactivate);
						}
					}
					mView->SetNeedsDisplay(TRUE);
				}
				else
				{
					for(int i=0; i < vwChildren->GetCount(); i++)
					{
						CPPMenuItem *menuItem = dynamic_cast<CPPMenuItem*>(vwChildren->GetNode(i));
						if(menuItem)
						{
							if(!menuItem->GetSubMenu() && menuItem->IsActive())
								menuItem->Message(cppControlDeactivate);
						}
					}
					mView->SetNeedsDisplay(TRUE);
				}
			}
		}
	}
}

void CPPMenu::ScrollWheel(CGEventRef cgEvent)
{
}

void CPPMenu::KeyDown(CGEventRef cgEvent)
{
}

void CPPMenu::KeyUp(CGEventRef cgEvent)
{
}

CPPMenuItem* CPPMenu::AddMenuItem(const char *str, CPPNode *trg, int msg, void *data, const char *htKy, int mdKy)
{
	CPPMenuItem *mItem = new CPPMenuItem(); if(!mItem) return NULL;
	
	mItem->InitWithFrameAndLabel(CGRectMake(0,0,0,0),str,htKy,ModifierKeyToCharacter(mdKy));
	AddViewToWindowContent(mItem);
	
	// set targets
	mItem->SetTarget(trg);
	mItem->SetMessage(msg);
	mItem->SetMessageData(data);
	mItem->SetHotKey(GetCharKeyCode(htKy));
	mItem->SetModifierKey(mdKy);
	
	//CGRect frm = GetWindowFrame();
	CGRect frm = GetContentFrame();
	CGSize itemSize = mItem->GetViewSize();
	if(frm.size.width < itemSize.width) frm.size.width = itemSize.width;
	mItem->SetOffset(GetOffset());
	AddOffset(itemSize.height);
	frm.size.height = lastOffset + 4.0;
	SetContentSize(frm.size);
	
	//Message(cppWindowDidResize,&frm);
	Message(cppWindowDidResize);
	
	return mItem;
}

void CPPMenu::AddOffset(CGFloat off)
{
	lastOffset += off;
}

CGFloat CPPMenu::GetOffset(void)
{
	return lastOffset;
}



/*****************************************************************************/
// CPPMenuItem C++ class functions

CPPMenuItem::CPPMenuItem()
{
	stateLabel = NULL;
	itemLabel = NULL;
	hotkeyLabel = NULL;
	subMenu = NULL;
	
	menuBarItem = FALSE;
	isEnabled = TRUE;
	menuState = 0;
	
	target = NULL;
	message = cppGenericMessage;
	msgData = NULL;
	
	modKey = cppModifierNone;
	hotKey = -1;
}

CPPMenuItem::~CPPMenuItem()
{
}

void CPPMenuItem::SetOffset(CGFloat off)
{
	offset = off;
}

CGFloat CPPMenuItem::GetOffset(void)
{
	return offset;
}

void CPPMenuItem::SetTarget(CPPNode *trg)
{
	target = trg;
}

CPPNode* CPPMenuItem::GetTarget(void)
{
	return target;
}

void CPPMenuItem::SetMessage(int msg)
{
	message = msg;
}

int CPPMenuItem::GetMessage(void)
{
	return message;
}

void CPPMenuItem::SetMessageData(void *mDat)
{
	msgData = mDat;
}

void* CPPMenuItem::GetMessageData(void)
{
	return msgData;
}

void CPPMenuItem::SetHotKey(int hk)
{
	hotKey = hk;
}

int CPPMenuItem::GetHotKey(void)
{
	return hotKey;
}

void CPPMenuItem::SetModifierKey(int mk)
{
	modKey = mk;
}

int CPPMenuItem::GetModifierKey(void)
{
	return modKey;
}

void CPPMenuItem::SetIsMenuBarItem(bool flag)
{
	menuBarItem = flag;
}

bool CPPMenuItem::IsMenuBarItem(void)
{
	return menuBarItem;
}

void CPPMenuItem::SetIsEnabled(bool flag)
{
	isEnabled = flag;
}

bool CPPMenuItem::IsEnabled(void)
{
	return isEnabled;
}

void CPPMenuItem::SetMenuState(int state)
{
	menuState = state;
}

int CPPMenuItem::GetMenuState(void)
{
	return menuState;
}

const char* CPPMenuItem::GetName()
{
	return itemLabel->GetString();
}

CPPMenu* CPPMenuItem::AddSubMenu(void)
{
	subMenu = new CPPMenu(); if(!subMenu) return NULL;
	
	unsigned int msk = CPPBorderlessWindowMask | CPPUnscaledWindowMask | CPPClosableWindowMask;
	CGRect mpRect = CGRectMake(0,0,0,0);
	subMenu->InitWithContentRect(mpRect, msk);
	subMenu->SetParent(this);

	subMenu->SetContentRect(mpRect);
	subMenu->SetFloatingPanel(TRUE);
	//subMenu->SetBackgroundColor(0.7f,0.7f,0.7f,1.0f);
	subMenu->SetBecomesKeyOnlyIfNeeded(TRUE);
	
	if(!IsMenuBarItem())
	{
		hotkeyLabel->SetString("▶");
		CGPoint origin = hotkeyLabel->GetFrameOrigin();
		origin.y += 1.0;
		//hotkeyLabel->SetFrameOrigin(origin);
		hotkeyLabel->SetViewOrigin(origin);
	}
	
	return subMenu;
}

CPPMenu* CPPMenuItem::GetSubMenu(void)
{
	return subMenu;
}

bool CPPMenuItem::Message(int msg, void *data)
{
	switch(msg)
	{
		case cppControlActivate:
		{	
			SetIsActive(TRUE);
			if(stateLabel) stateLabel->SetHilightColor();
			if(itemLabel) itemLabel->SetHilightColor();
			if(hotkeyLabel) hotkeyLabel->SetHilightColor();
			
			if(subMenu)
			{
				CGPoint origin;
				CGPoint wndOrigin = GetWindowContentFrame().origin;
				CGSize wndSize = GetWindowContentRect().size;
				CGPoint mnOrigin = GetViewOrigin();
				CGSize mnSize = GetViewSize();
				
				if(IsMenuBarItem())
					//origin = CGPointMake(wndOrigin.x + mnOrigin.x, wndOrigin.y + mnOrigin.y);
					origin = CGPointMake(wndOrigin.x + mnOrigin.x, wndOrigin.y + mnSize.height + 1.0);
				else
					origin = CGPointMake(wndOrigin.x + mnOrigin.x + mnSize.width, wndOrigin.y + mnOrigin.y - 4.0);
				
				subMenu->SetFrameTopLeftPoint(origin);
				subMenu->OpenPanel();
			}
			break;
		}
		case cppControlDeactivate:
		{	
			SetIsActive(FALSE);
			if(stateLabel) stateLabel->SetNormalColor();
			if(itemLabel) itemLabel->SetNormalColor();
			if(hotkeyLabel) hotkeyLabel->SetNormalColor();
			
			if(subMenu) subMenu->Message(cppControlDeactivate);
			break;
		}
		case cppResizeContent:
		{
			CGRect *cntRect = (CGRect*)data;
			if(cntRect)
			{
				if(IsMenuBarItem())
				{
					CGPoint vOrigin = GetViewOrigin();
					vOrigin.y = cntRect->origin.y;
					SetViewOrigin(vOrigin);
				}
				else
				{
					CGSize vSize = GetViewSize();
					if(vSize.width < cntRect->size.width) vSize.width = cntRect->size.width;
					SetViewSize(vSize);
					//SetFrameSize(vSize);
					
					CGPoint vOrigin = GetViewOrigin();
					//vOrigin.y = cntRect->size.height - GetOffset();
					vOrigin.y = GetOffset();
					SetViewOrigin(vOrigin);
					//SetFrameOrigin(vOrigin);
					
					if(hotkeyLabel)
					{
						CGPoint lOrigin = hotkeyLabel->GetViewOrigin();
						CGFloat w = hotkeyLabel->GetViewSize().width;
						
						lOrigin.x = vOrigin.x + vSize.width - w - 4.0;
						hotkeyLabel->SetViewOrigin(lOrigin);
						//hotkeyLabel->SetFrameOrigin(lOrigin);
					}
				}
			}
			break;
		}
	}
	
	return TRUE;
}

void CPPMenuItem::BlinkMenuSelection(void)
{
	Message(cppControlDeactivate);
	Draw();
	
	double blinkTimer;
	
	StartTimer(blinkTimer);
	while(GetElapsedTime(blinkTimer) < 0.1);
	Message(cppControlActivate);
	Draw();
	
	StartTimer(blinkTimer);
	while(GetElapsedTime(blinkTimer) < 0.1);
	Message(cppControlDeactivate);
	Draw();
}

// events

void CPPMenuItem::LeftMouseDown(CGEventRef cgEvent)
{
}

void CPPMenuItem::LeftMouseUp(CGEventRef cgEvent)
{
	if(!IsMenuBarItem() && IsActive())
	{
		CGPoint mPos = CGEventGetLocation(cgEvent);
		if(MouseInViewRect(mPos))
		{
			if(!GetSubMenu())
			{
				BlinkMenuSelection();
				CPPApp->SendMessage(target,GetMessage(),GetMessageData());
			}
			else
			{
				Message(cppControlDeactivate);
			}
		}
	}
}

void CPPMenuItem::KeyDown(CGEventRef cgEvent)
{
	CGEventFlags flags = CGEventGetFlags(cgEvent);
	if(ModifierKey(modKey,flags))
	{
		if(GetEventKeyCode(cgEvent) == hotKey)
		{
			// handle the hot key
			CPPApp->SendMessage(target,GetMessage(),GetMessageData());
		}
	}
}
