/*
 *  CPPNode.cpp
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/19/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */


#include "CPPNode.h"

CPPNode::CPPNode()
{
	parent = NULL;
	children = new CPPNodeList();
}

CPPNode::~CPPNode()
{
	Remove();
	while(children->GetCount() > 1)
	{
		CPPNode *ch = children->GetNode(0);
		if(ch) delete ch;
	}
	if(children) delete children;
}

void CPPNode::SetParent(CPPNode *pr)
{
	if(pr == NULL && parent != NULL)
	{
		parent->GetChildren()->Remove(this);
		parent = NULL;
	}
	else if(pr != NULL && pr->GetParent() != this)
	{
		parent = pr;
		parent->GetChildren()->Append(this);
	}
}

CPPNode* CPPNode::GetParent(void)
{
	return parent;
}

CPPNodeList* CPPNode::GetChildren(void)
{
	return children;
}

CPPNode* CPPNode::GetChild(void)
{
	if(children->GetCount() < 1) return NULL;
	
	return children->GetNode(0);
}

CPPNode* CPPNode::GetPrev(void)
{
	if(!parent) return NULL;
	
	int index = parent->GetChildren()->GetIndex(this);
	index--;
	
	if(index < 0) return NULL;
	
	return parent->GetChildren()->GetNode(index);
}

CPPNode* CPPNode::GetNext(void)
{
	if(!parent) return NULL;
	
	int index = parent->GetChildren()->GetIndex(this);
	index++;
	
	if(index > parent->GetChildren()->GetCount()) return NULL;
	
	return parent->GetChildren()->GetNode(index);
}

CPPNode* CPPNode::GetLastChild(void)
{
	if(children->GetCount() < 1) return NULL;
	
	return children->GetNode(children->GetCount()-1);
}

void CPPNode::Remove(void)
{
	if(parent)
	{
		parent->GetChildren()->Remove(this);
		SetParent(NULL);
	}
}

void CPPNode::Insert(CPPNode *n)
{
	if(!n) return;
	CPPNode *pr = n->GetParent(); if(!pr) return;
	
	CPPNodeList *ch = pr->GetChildren();
	int index = ch->GetIndex(n);
	ch->Insert(index,this);
	SetParent(pr);
}

void CPPNode::Draw(void)
{
}

bool CPPNode::Message(int msg, void *data)
{
	return TRUE;
}

void CPPNode::SendMessage(CPPNode *n, int m, void *d)
{
	nodeTrg = n;
	nodeMsg.msg = m;
	nodeMsg.data = d;
}

void CPPNode::ClearMessage(void)
{
	nodeTrg = NULL;
	nodeMsg.msg = 0;
	nodeMsg.data = NULL;
}

bool CPPNode::ValidMessage(void)
{
	return nodeMsg.msg > 0;
}


