/*
 *  CPPOpenGL3DViewPort.cpp
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/2/12.
 *  Copyright 2012 Dan Libisch. All rights reserved.
 *
 */

#include "CPPOpenGL3DViewPort.h"


CPPOpenGL3DViewPort::CPPOpenGL3DViewPort()
{
	// set view defaults
	SetFieldOfView(50.0f);
	SetNearClip(0.001f);
	SetFarClip(1000.0f);
	
	// set navigation defaults
	SetViewPortRotation(-45.0f,25.0f);
	SetViewPortPosition(0,0,-10.0f);

	CGColorRef tClr = CGColorCreateGenericRGB(0.35f,0.35f,0.35f,1.0f);
	CGColorRef bClr = CGColorCreateGenericRGB(0.0f,0.0f,0.0f,1.0f);
	SetBackgroundColors(tClr, bClr);

	grid = new Grid();
	grid->SetParent(this);

	// default view margins
	SetViewMargins(0,0,0,0);
	
	navigating = FALSE;
	mDragStart = CGPointMake(0,0);
}

CPPOpenGL3DViewPort::~CPPOpenGL3DViewPort()
{
	Remove();
}

// OpenGL functions
void CPPOpenGL3DViewPort::InitOpenGL(void)
{
    // set up gizmo
	SetPerspectiveProjection(vwRect,fieldOfView,nearClip,farClip);
	
	glShadeModel(GL_SMOOTH);
	glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
	glClearDepth(1.0f);
	
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_COLOR_MATERIAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_PERSPECTIVE_CORRECTION_HINT);
	glEnable(GL_CULL_FACE);
}

void CPPOpenGL3DViewPort::SetViewPortRotation(GLfloat x, GLfloat y)
{
	viewRotateX = x;
	viewRotateY = y;
}

void CPPOpenGL3DViewPort::SetViewPortPosition(GLfloat x, GLfloat y, GLfloat z)
{
	viewPositionX = x;
	viewPositionY = y;
	viewPositionZ = z;
}

void CPPOpenGL3DViewPort::SetFieldOfView(GLdouble fov)
{
	fieldOfView = fov;
}

void CPPOpenGL3DViewPort::SetNearClip(GLdouble nc)
{
	nearClip = nc;
}

void CPPOpenGL3DViewPort::SetFarClip(GLdouble fc)
{
	farClip = fc;
}

void CPPOpenGL3DViewPort::Draw(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	DrawViewPortBackground();
	
	SetPerspectiveProjection(vwRect,fieldOfView,nearClip,farClip);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// set view navigation
	glTranslatef(viewPositionX,viewPositionY,viewPositionZ);
	glRotatef(viewRotateY,1,0,0);
	glRotatef(viewRotateX,0,1,0);
	
	// set opengl options
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	
	grid->Draw();
	
	drawCube();
	
	glFlush();
	FlushBuffer();
}

void CPPOpenGL3DViewPort::DrawViewPortBackground(void)
{
	glPushAttrib(GL_ENABLE_BIT | GL_DEPTH_BUFFER_BIT | GL_TRANSFORM_BIT );
	
	glDisable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glBegin(GL_QUADS);
	
	//top color
	const CGFloat *tClr = CGColorGetComponents(bgrTColor);
	glColor3f(tClr[0], tClr[1], tClr[2]);
	glVertex2f(1.0,1.0);
	glVertex2f(-1.0, 1.0);
	
	//bottom color
	const CGFloat *bClr = CGColorGetComponents(bgrBColor);
	glColor3f(bClr[0], bClr[1], bClr[2]);
	glVertex2f(-1.0, -1.0);
	glVertex2f(1.0,-1.0);
	
	glEnd();
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glEnable(GL_DEPTH_TEST);
	
	glPopAttrib();
}

void CPPOpenGL3DViewPort::SetBackgroundColors(CGColorRef top, CGColorRef bottom)
{
	bgrTColor = top;
	bgrBColor = bottom;
}

bool CPPOpenGL3DViewPort::Message(int msg, void *data)
{
	switch(msg)
	{
		case cppResizeContent:
		{
			mDragStart = CGPointMake(0,0);
			CGRect *cntRect = (CGRect*)data;
			if(cntRect)
			{
				ResizeViewRect(*cntRect);
			}
			break;
		}
	}
	
	return TRUE;
}


// OpenGL view events
void CPPOpenGL3DViewPort::LeftMouseDown(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	if(MouseInViewRect(mPos))
	{
		//printf("CPPOpenGL3DViewPort::LeftMouseDown \n");
		mDragStart = mPos;
		CGEventFlags flags = CGEventGetFlags(cgEvent);
		if(ModifierKey(cppAlternateKey,flags))
		{
			mxLast = 0;
			myLast = 0;
			navigating = TRUE;
		}
	}
}

void CPPOpenGL3DViewPort::RightMouseDown(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	if(MouseInViewRect(mPos))
	{
		//printf("CPPOpenGL3DViewPort::RightMouseDown \n");
		mDragStart = mPos;
		CGEventFlags flags = CGEventGetFlags(cgEvent);
		if(ModifierKey(cppAlternateKey,flags))
		{
			mxLast = 0;
			myLast = 0;
			navigating = TRUE;
		}
	}
}

void CPPOpenGL3DViewPort::OtherMouseDown(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	if(MouseInViewRect(mPos))
	{
		//printf("CPPOpenGL3DViewPort::OtherMouseDown \n");
		mDragStart = mPos;
		CGEventFlags flags = CGEventGetFlags(cgEvent);
		if(ModifierKey(cppAlternateKey,flags))
		{
			mxLast = 0;
			myLast = 0;
			navigating = TRUE;
		}
	}
}

void CPPOpenGL3DViewPort::LeftMouseUp(CGEventRef cgEvent)
{
	//printf("CPPOpenGL3DViewPort::LeftMouseUp \n");
	navigating = FALSE;
	mDragStart = CGPointMake(0,0);
}

void CPPOpenGL3DViewPort::RightMouseUp(CGEventRef cgEvent)
{
	//printf("RCPPOpenGLView::RightMouseUp \n");
	
	navigating = FALSE;
	mDragStart = CGPointMake(0,0);
}

void CPPOpenGL3DViewPort::OtherMouseUp(CGEventRef cgEvent)
{
	//printf("CPPOpenGL3DViewPort::OtherMouseUp \n");
	
	navigating = FALSE;
	mDragStart = CGPointMake(0,0);
}

void CPPOpenGL3DViewPort::MouseMoved(CGEventRef cgEvent)
{
	//printf("CPPOpenGL3DViewPort::MouseMoved \n");
}

void CPPOpenGL3DViewPort::ScrollWheel(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	if(MouseInViewRect(mPos))
	{
		//printf("CPPOpenGL3DViewPort::ScrollWheel %i, %i, %i \n", mxDelta, myDelta, mwDelta);
		CGEventFlags flags = CGEventGetFlags(cgEvent);
		if(ModifierKey(cppAlternateKey,flags)) viewPositionZ -= (viewPositionZ * 0.001f) * mwDelta;
		else viewPositionZ -= (viewPositionZ * 0.01f) * mwDelta;
	}
}

void CPPOpenGL3DViewPort::LeftMouseDragged(CGEventRef cgEvent)
{
	if(MouseInViewRect(mDragStart))
	{
		if(navigating)
		{
			//printf("CPPOpenGL3DViewPort::LeftMouseDragged Navigating %i, %i \n", mxDelta, myDelta);
			viewRotateX += mxDelta * 0.25f;
			viewRotateY += myDelta * 0.25f;
		}
	}
}

void CPPOpenGL3DViewPort::RightMouseDragged(CGEventRef cgEvent)
{
	if(MouseInViewRect(mDragStart))
	{
		if(navigating)
		{
			//printf("CPPOpenGL3DViewPort::RightMouseDragged Navigating  %i, %i \n", mxDelta, myDelta);
			viewPositionZ += mxDelta * 0.005f;
			viewPositionZ -= myDelta * 0.005f;
		}
		else
		{
			CGPoint mPos = CGEventGetLocation(cgEvent);
			if(MouseInViewRect(mPos))
			{
				//printf("CPPOpenGL3DViewPort::RightMouseDragged \n");
			}
		}
	}
}

void CPPOpenGL3DViewPort::OtherMouseDragged(CGEventRef cgEvent)
{
	if(MouseInViewRect(mDragStart))
	{
		if(navigating)
		{
			//printf("CPPOpenGL3DViewPort::OtherMouseDragged Navigating  %i, %i \n", mxDelta, myDelta);
			float panScale = abs(viewPositionZ) * 0.001f;
			if(panScale < 0.0001f) panScale = 0.0001f;
			viewPositionX += mxDelta * panScale;
			viewPositionY -= myDelta * panScale;
		}
		else
		{
			CGPoint mPos = CGEventGetLocation(cgEvent);
			if(MouseInViewRect(mPos))
			{
				//printf("CPPOpenGL3DViewPort::OtherMouseDragged \n");
			}
		}
	}
}

void CPPOpenGL3DViewPort::KeyDown(CGEventRef cgEvent)
{
	//printf("CPPOpenGL3DViewPort::KeyDown \n");
	switch(GetEventKeyCode(cgEvent))
	{
		case CPP_KEY_G:
			grid->SetVisibility(!grid->IsVisible());
			break;
	}
}

void CPPOpenGL3DViewPort::KeyUp(CGEventRef cgEvent)
{
	//printf("CPPOpenGL3DViewPort::KeyUp \n");
}

void CPPOpenGL3DViewPort::FlagsChanged(CGEventRef cgEvent)
{
	//printf("PPOpenGLView::FlagsChanged \n");
}
