/*
 *  CPPResponder.cpp
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/4/12.
 *  Copyright 2012 Dan Libisch. All rights reserved.
 *
 */

#include "CPPResponder.h"

CPPResponder::CPPResponder()
{
	SetRespondsToEvents(TRUE);
}

CPPResponder::~CPPResponder()
{
}

bool CPPResponder::Message(int msg, void *data)
{
	return TRUE;
}

void CPPResponder::SetRespondsToEvents(bool flag)
{
	respond = flag;
}

bool CPPResponder::IsResponder(void)
{
	return respond;
}

bool CPPResponder::DispatchCGEvents(CGEventRef cgEvent)
{
	for(int i=0; i < children->GetCount(); i++)
	{
		CPPResponder *ch = dynamic_cast<CPPResponder*>(children->GetNode(i));
		if(ch && ch->IsResponder())
		{
			if(!ch->DispatchCGEvents(cgEvent)) return FALSE;
		}
	}
	return HandleEvents(cgEvent);
}

bool CPPResponder::HandleEvents(CGEventRef cgEvent)
{
	CGEventType type = CGEventGetType(cgEvent);
	
	mxDelta = GetMouseDeltaX(cgEvent);
	myDelta = GetMouseDeltaY(cgEvent);
	mwDelta = GetMouseDeltaW(cgEvent);
	mClick = GetMouseClickCount(cgEvent);
	
	switch(type)
	{
		case kCGEventLeftMouseDown:
			LeftMouseDown(cgEvent);
			break;
		case kCGEventRightMouseDown:
			RightMouseDown(cgEvent);
			break;
		case kCGEventOtherMouseDown:
			OtherMouseDown(cgEvent);
			break;
		case kCGEventLeftMouseUp:
			LeftMouseUp(cgEvent);
			break;
		case kCGEventRightMouseUp:
			RightMouseUp(cgEvent);
			break;
		case kCGEventOtherMouseUp:
			OtherMouseUp(cgEvent);
			break;
		case kCGEventMouseMoved:
			MouseMoved(cgEvent);
			break;
		case kCGEventScrollWheel:
			ScrollWheel(cgEvent);
			break;
		case kCGEventLeftMouseDragged:
			LeftMouseDragged(cgEvent);
			break;
		case kCGEventRightMouseDragged:
			RightMouseDragged(cgEvent);
			break;
		case kCGEventOtherMouseDragged:
			OtherMouseDragged(cgEvent);
			break;
		case kCGEventKeyDown:
			KeyDown(cgEvent);
			break;
		case kCGEventKeyUp:
			KeyUp(cgEvent);
			break;
		case kCGEventFlagsChanged:
			FlagsChanged(cgEvent);
			break;
	}
	
	return TRUE;
}

void CPPResponder::LeftMouseDown(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	//printf("mPos.x = %f, mPos.y = %f\n",mPos.x, mPos.y);
}

void CPPResponder::RightMouseDown(CGEventRef cgEvent)
{
}

void CPPResponder::OtherMouseDown(CGEventRef cgEvent)
{
}

void CPPResponder::LeftMouseUp(CGEventRef cgEvent)
{
}

void CPPResponder::RightMouseUp(CGEventRef cgEvent)
{
}

void CPPResponder::OtherMouseUp(CGEventRef cgEvent)
{
}

void CPPResponder::MouseMoved(CGEventRef cgEvent)
{
	CGPoint mPos = CGEventGetLocation(cgEvent);
	//printf("mPos.x = %f, mPos.y = %f\n",mPos.x, mPos.y);
}

void CPPResponder::ScrollWheel(CGEventRef cgEvent)
{
}

void CPPResponder::LeftMouseDragged(CGEventRef cgEvent)
{
}

void CPPResponder::RightMouseDragged(CGEventRef cgEvent)
{
}

void CPPResponder::OtherMouseDragged(CGEventRef cgEvent)
{
}

void CPPResponder::KeyDown(CGEventRef cgEvent)
{
	CGEventFlags flags = CGEventGetFlags(cgEvent);
	if(ModifierKey(cppCommandAlternateKey,flags))
	{
		switch(GetEventKeyCode(cgEvent))
		{
			case CPP_KEY_B:
				Message(cppGenericMessage);
				break;
		}		
	}
}

void CPPResponder::KeyUp(CGEventRef cgEvent)
{
}

void CPPResponder::FlagsChanged(CGEventRef cgEvent)
{
}
