/*
 *  Grid.cpp
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/2/12.
 *  Copyright 2012 Dan Libisch. All rights reserved.
 *
 */

#include "Grid.h"

Grid::Grid()
{
    // Default grid 100*100
    width = 10;
    step = 1;
    group = 10;
	scale = 1.0f;

	displayAxis = TRUE;
	visible = TRUE;
}

Grid::~Grid()
{
	
}

void Grid::Draw(void)
{
    if(visible)
	{
		// Draw Grid
		for(int i = -width; i <= width; i+=step)
		{
			if(i % group == 0)
			{
				glColor3f(0.8, 0.8, 0.8);
				glLineWidth(1.0);
			}
			else
			{
				glColor3f(0.4, 0.4, 0.4);
				glLineWidth(1.0);
			}
			
			if(displayAxis && i == 0)
			{
				glBegin(GL_LINES);
				glVertex3f(i*scale,0,-width*scale);
				glVertex3f(i*scale,0,0);
				glEnd();
				
				glBegin(GL_LINES);
				glVertex3f(i*scale,0,3*scale);
				glVertex3f(i*scale,0,width*scale);
				glEnd();
				
				glBegin(GL_LINES);
				glVertex3f(-width*scale,0,i*scale);
				glVertex3f(0,0,i*scale);
				glEnd();
				
				glBegin(GL_LINES);
				glVertex3f(3*scale,0,i*scale);
				glVertex3f(width*scale,0,i*scale);
				glEnd();
				
				// X axis
				glColor3f(0.6, 0.0, 0.0);
				glBegin(GL_LINES);
				glVertex3f(0,0,i*scale);
				glVertex3f(3*scale,0,i*scale);
				glEnd();
				
				// Z axis
				glColor3f(0.0, 0.0, 0.6);
				glBegin(GL_LINES);
				glVertex3f(i*scale,0,0);
				glVertex3f(i*scale,0,3*scale);
				glEnd();
			}
			else
			{
				glBegin(GL_LINES);
				glVertex3f(i*scale,0,-width*scale);
				glVertex3f(i*scale,0,width*scale);
				glEnd();
				
				glBegin(GL_LINES);
				glVertex3f(-width*scale,0,i*scale);
				glVertex3f(width*scale,0,i*scale);
				glEnd();
			}
		}
		
		if(displayAxis)
		{
			// Y axis
			glColor3f(0.0, 0.6, 0.0);
			glLineWidth(1.0);
			
			glBegin(GL_LINES);
			glVertex3f(0,0,0);
			glVertex3f(0,3*scale,0);
			glEnd();
		}
	}
}

void Grid::SetWidth(int w)
{
	width = w;
}

void Grid::SetStep(int s)
{
	step = s;
}

void Grid::SetGroup(int g)
{
	group = g;
}

void Grid::SetScale(float sca)
{
	scale = sca;
}

void Grid::EnableAxis(bool flag)
{
	displayAxis = flag;
}

void Grid::SetVisibility(bool flag)
{
	visible = flag;
}

bool Grid::IsVisible(void)
{
	return visible;
}
