//
//  CPPApplication.mm
//  CPPAppKitTest
//
//  Created by Dan Libisch on 2/25/12.
//  Copyright 2012 Dan Libisch. All rights reserved.
//

#include <AppKit/AppKit.h>

#include "CPPApplication.h"


// NSEvent storage pointer
static NSEvent *nsEvent;

/*****************************************************************************/
// Objective-C  Class CppNsApplication : NSApplication

@interface CppNsApplication : NSApplication
{
	CPPApplication *cppApp;
	NSString *appName;
}
- (void)setApplicationName;
- (NSString*)getApplicationName;
- (void)terminateApplication:(id)sender;
- (void)setCppApplicationPointer:(CPPApplication*)capp;
- (CPPApplication*)getCppApplicationPointer;
@end


@implementation CppNsApplication
- (void)setCppApplicationPointer:(CPPApplication*)capp
{ cppApp = capp; }

- (CPPApplication*)getCppApplicationPointer
{ return cppApp; }

- (IBAction)openAboutPanel:(id)sender
{ NSDictionary *options = [NSDictionary dictionaryWithObjectsAndKeys:
			   @"0.1", @"Version",
			   @"CPPAppKitTest", @"ApplicationName",
			   @"Copyright 2012, Dan Libisch", @"Copyright",
			   @"CPPAppKitTest v0.1", @"ApplicationVersion",
			   nil];
    [NSApp orderFrontStandardAboutPanelWithOptions:options];
}

- (void)setApplicationName
{ appName = [[NSProcessInfo processInfo] processName]; }

- (NSString*)getApplicationName
{ return appName; }

- (void)terminateApplication:(id)sender
{ cppApp->TerminateApplication(sender); }

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{ if(!cppApp->Message(cppApplicationShouldTerminate)) return NSTerminateCancel;
	else return NSTerminateNow; }

- (void)applicationWillTerminate:(NSNotification *)aNotification
{ cppApp->Message(cppApplicationWillTerminate); }
@end





/*****************************************************************************/
// CPPApplication Objective-C call functions

void CPPApplication::InitSharedApplication(void)
{
	nsObjPtr = [CppNsApplication sharedApplication];
	[nsObjPtr setCppApplicationPointer:this];
	[nsObjPtr setApplicationName];
	SetDelegate(nsObjPtr);
}

void CPPApplication::InitMainMenu(void)
{
    id appName = [nsObjPtr getApplicationName];
	
	NSMenu *menubar = [[[NSMenu alloc] init] autorelease];
    [nsObjPtr setMainMenu:menubar];
	
	// standard application main menu
	NSMenuItem *appMenuItem = [menubar addItemWithTitle:@"" action:NULL keyEquivalent:@""];
	
    NSMenu *appMenu = [[NSMenu alloc] init];
    [appMenuItem setSubmenu:appMenu];
	
    [appMenu addItemWithTitle:[NSString stringWithFormat:@"About %@", appName] action:@selector(openAboutPanel:) keyEquivalent:@""];
    [appMenu addItem:[NSMenuItem separatorItem]];
    
	NSMenu *servicesMenu = [[NSMenu alloc] init];
    [NSApp setServicesMenu:servicesMenu];
    [[appMenu addItemWithTitle:@"Services" action:NULL keyEquivalent:@""] setSubmenu:servicesMenu];
	
    [appMenu addItem:[NSMenuItem separatorItem]];
    
	[appMenu addItemWithTitle:[NSString stringWithFormat:@"Hide %@", appName] action:@selector(hide:) keyEquivalent:@"h"];
    [[appMenu addItemWithTitle:@"Hide Others" action:@selector(hideOtherApplications:) keyEquivalent:@"h"] setKeyEquivalentModifierMask:NSCommandKeyMask];
    [appMenu addItemWithTitle:@"Show All" action:@selector(unhideAllApplications:) keyEquivalent:@""];
	
    [appMenu addItem:[NSMenuItem separatorItem]];
	
    [appMenu addItemWithTitle:[NSString stringWithFormat:@"Quit %@", appName] action:@selector(terminateApplication:) keyEquivalent:@"q"];
	
    NSMenuItem *windowMenuItem = [menubar addItemWithTitle:@"" action:NULL keyEquivalent:@""];
    NSMenu *windowMenu = [[NSMenu alloc] initWithTitle:@"Window"];
    [NSApp setWindowsMenu:windowMenu];
    [windowMenuItem setSubmenu:windowMenu];
	
    [windowMenu addItemWithTitle:@"Miniaturize" action:@selector(performMiniaturize:) keyEquivalent:@"m"];
    [windowMenu addItemWithTitle:@"Zoom" action:@selector(performZoom:) keyEquivalent:@""];
    [windowMenu addItem:[NSMenuItem separatorItem]];
    [windowMenu addItemWithTitle:@"Bring All to Front" action:@selector(arrangeInFront:) keyEquivalent:@""];
	
}

void CPPApplication::FinishLaunching(void)
{
	[nsObjPtr finishLaunching];
	
	NSArray *scrns = [NSScreen screens];
	int i, cnt = [scrns count];
	for(i=0; i<cnt; i++)
	{
		id scrn = [scrns objectAtIndex:i];
		NSRect scrFrame = [scrn frame];
		printf("screen %i: x = %f, y = %f, w = %f, h = %f\n", i, scrFrame.origin.x, scrFrame.origin.y, scrFrame.size.width, scrFrame.size.height);
	}
	printf("\n");
}

id CPPApplication::GetApplicationName(void)
{
	return [nsObjPtr getApplicationName];
}

void CPPApplication::SetDelegate(id del)
{
	[nsObjPtr setDelegate:del];
}

CGEventRef CPPApplication::PollNextEvent(void)
{
	CGEventRef cgEvent;
	
	nsEvent = [nsObjPtr nextEventMatchingMask: NSAnyEventMask untilDate:[NSDate distantFuture] inMode: NSDefaultRunLoopMode dequeue: YES];
	if(nsEvent)
	{
		cgEvent = [nsEvent CGEvent];
		
		// convert the CGEvent location
		//NSPoint eventPt = [NSEvent mouseLocation];
		//NSPoint eventPt = [nsEvent locationInWindow];
		//CGFloat wndHt = mainWindow->GetContentRect().size.height; 
		//eventPt.y = wndHt - eventPt.y;
		//CGPoint location = NSPointToCGPoint(eventPt);
		//CGEventSetLocation(cgEvent,location);
		
		//CGPoint mPt = CGEventGetLocation(cgEvent);
		//printf("mouseX = %f, mouseY = %f\n",mPt.x,mPt.y);
	}
	
	return cgEvent;
}

void CPPApplication::ForwardEventToNSApplication(void)
{
	[nsObjPtr sendEvent:nsEvent];
}

void CPPApplication::TerminateApplication(id sender)
{
	[NSApp terminate:sender];
}

