/*
 *  CPPFileSelector.mm
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/11/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */

#include <AppKit/AppKit.h>

#include "CPPFileSelector.h"


int CallFileSelectionDialog(int type, int flag, char **fName)
{
	switch(flag)
	{
		case FILE_OPEN:
			return CallNSOpenPanel(type,fName);
		case FILE_SAVE:
			return CallNSSavePanel(type,fName);
	}
	
	return 0;
}

//Create open panel dialog
int CallNSOpenPanel(int type, char **fName)
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	NSWindow *keyWindow = [NSApp keyWindow]; // get the main window
	
	NSOpenPanel *open = [NSOpenPanel openPanel];
	NSString *pathName;
	
	[open setResolvesAliases: true]; //Enable alias resolving
	[open setAllowsMultipleSelection: false]; //Disable multiple selection
	
	switch(type)
	{
		case FILE_SELECTION:
			[open setCanChooseDirectories: false]; //Disable folder selection
			[open setCanChooseFiles: true]; //Enable file selection
			break;
		case DIR_SELECTION:
			[open setCanChooseDirectories: true]; //Enable folder selection
			[open setCanChooseFiles: false]; //Disable file selection
			break;
	}
	
	//Display open panel
	int result = [open runModal];
	if(result == NSOKButton)
	{
		switch(type)
		{
			case FILE_SELECTION:
				pathName = [open filename];
				break;
			case DIR_SELECTION:
				pathName = [open directory];
				break;
		}
		
		*fName = (char *)malloc(strlen([pathName UTF8String])+1);
		strcpy(*fName, [pathName UTF8String]);
		
		[keyWindow makeKeyWindow]; // restore focus to the main window
		[pool release];
		return 1;
	}
	
	[keyWindow makeKeyWindow]; // restore focus to the main window
	[pool release];
	return 0;
}

//Create save panel dialog
int CallNSSavePanel(int type, char **fName)
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	NSWindow *keyWindow = [NSApp keyWindow]; // get the main window
	
	NSSavePanel *save = [NSSavePanel savePanel];
	NSString *pathName;
	
	switch(type)
	{
		case FILE_SELECTION:
			[save setCanCreateDirectories: false]; //Disable folder creation
			break;
		case DIR_SELECTION:
			[save setCanCreateDirectories: true]; //Enable folder creation
			break;
	}
	
	//Display save panel
	int result = [save runModal];
	if(result == NSOKButton)
	{
		switch(type)
		{
			case FILE_SELECTION:
				pathName = [save filename];
				break;
			case DIR_SELECTION:
				pathName = [save directory];
				break;
		}
		*fName = (char *)malloc(strlen([pathName UTF8String])+1);
		strcpy(*fName, [pathName UTF8String]);
		
		[keyWindow makeKeyWindow]; // restore focus to the main window
		[pool release];
		return 1;
	}
	
	[keyWindow makeKeyWindow]; // restore focus to the main window
	[pool release];
	return 0;
}

