/*
 *  CPPGlobals.mm
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/17/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */

#include <AppKit/AppKit.h>
#include "CPPGlobals.h"


bool PointInRect(CGPoint p, CGRect r)
{
	if(p.x > r.origin.x && p.y > r.origin.y && p.x < r.origin.x + r.size.width && p.y < r.origin.y + r.size.height) return TRUE;
	return FALSE;
}

CGRect GetPositionScreenFrame(CGPoint point)
{
	CGRect scrFrame;
	
	NSArray *scrns = [NSScreen screens];
	int i, cnt = [scrns count];
	for(i=0; i<cnt; i++)
	{
		id scrn = [scrns objectAtIndex:i];
		scrFrame = NSRectToCGRect([scrn frame]);
		scrFrame.origin.y = 0.0;
		
		if(PointInRect(point,scrFrame)) break;
	}
	
	return scrFrame;
}

id GetNSColor(CPPColor color)
{
	return [NSColor colorWithCalibratedRed:color.r green:color.g blue:color.b alpha:color.a]; 
}

id GetNSColor(CGFloat r, CGFloat g, CGFloat b, CGFloat a)
{
	return [NSColor colorWithCalibratedRed:r green:g blue:b alpha:a]; 
}

int CPPRunAlertPanel(const char *title, const char *msg, const char *btn1, const char *btn2, const char *btn3)
{
	NSString *t = GetNSString(title);
	NSString *m = GetNSString(msg);
	NSString *b1 = GetNSString(btn1);
	NSString *b2 = GetNSString(btn2);
	NSString *b3 = GetNSString(btn3);
	
	return NSRunAlertPanel(t,m,b1,b2,b3);
}

double CPPGetTimer(void)
{
	return [[NSDate date] timeIntervalSinceReferenceDate];
}

// UI fonts
id GetNSString(const char *str)
{
	return [NSString stringWithUTF8String:str];
}

id GetNSFont(int font, CGFloat size)
{
	switch(font)
	{
		case cppBoldSystemFont:
			return [NSFont boldSystemFontOfSize:size];
			
		case cppLabelFont:
			return [NSFont labelFontOfSize:size];
			
		case cppTitleBarFont:
			return [NSFont titleBarFontOfSize:size];
			
		case cppMenuFont:
			return [NSFont menuFontOfSize:size];
			
		case cppMenuBarFont:
			return [NSFont menuBarFontOfSize:size];
			
		case cppMessageFont:
			return [NSFont messageFontOfSize:size];
			
		case cppPaletteFont:
			return [NSFont paletteFontOfSize:size];
			
		case cppToolTipsFont:
			return [NSFont toolTipsFontOfSize:size];
			
		case cppControlContentFont:
			return [NSFont controlContentFontOfSize:size];
	}
	
	// default font
	return [NSFont systemFontOfSize:size];
}

CGFloat SystemFontSize(void)
{
	return [NSFont systemFontSize];
}

CGFloat SmallSystemFontSize(void)
{
	return [NSFont smallSystemFontSize];
}

CGFloat LabelFontSize(void)
{
	return [NSFont labelFontSize];
}

CGSize GetStringDrawingSize(int font, CGFloat size, const char *str, float padding)
{
	NSTextStorage *textStorage = [[[NSTextStorage alloc] initWithString:GetNSString(str)] autorelease];
	NSTextContainer *textContainer = [[[NSTextContainer alloc] initWithContainerSize: NSMakeSize(FLT_MAX, FLT_MAX)] autorelease];
	NSLayoutManager *layoutManager = [[[NSLayoutManager alloc] init] autorelease];
	
	[layoutManager addTextContainer:textContainer];
	
	[textStorage addLayoutManager:layoutManager];
	[textStorage addAttribute:NSFontAttributeName value:GetNSFont(font,size) range:NSMakeRange(0, [textStorage length])];
	
	[textContainer setLineFragmentPadding:padding];
	[textContainer setWidthTracksTextView:NO];
	[textContainer setHeightTracksTextView:NO];
	
	(void) [layoutManager glyphRangeForTextContainer:textContainer];
	NSSize textContainerSize = [layoutManager usedRectForTextContainer:textContainer].size;
	
	return NSSizeToCGSize(textContainerSize);
}

