/*
 *  CppNsOpenGLView.mm
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/16/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */

#include <AppKit/AppKit.h>
#include "CPPOpenGLView.h"


/*****************************************************************************/
// Objective-C  Class CppNsOpenGLView : NSOpenGLView

@interface CppNsOpenGLView : NSOpenGLView
{ CPPOpenGLView *cppGlVw; }
- (void)setCppNsOpenGLViewPointer:(CPPOpenGLView*)glvw;
- (CPPOpenGLView*)getCppNsOpenGLViewPointer;
@end


@implementation CppNsOpenGLView
- (void)setCppNsOpenGLViewPointer:(CPPOpenGLView*)glvw
{ cppGlVw = glvw; }

- (CPPOpenGLView*)getCppNsOpenGLViewPointer
{ return cppGlVw; }

- (BOOL)isFlipped
{ return YES; }
@end



/*****************************************************************************/
// CPPOpenGLView Objective-C call functions

id CPPOpenGLView::InitWithFrame(CGRect cRect, bool view3D)
{
	NSRect contentRect = NSRectFromCGRect(cRect);
	nsObjPtr = [[[CppNsOpenGLView alloc] initWithFrame:contentRect pixelFormat:[NSOpenGLView defaultPixelFormat]] autorelease];
	
	if(view3D) Set3DViewPixelFormat();

	MakeCurrentContext();
	SetCppNsOpenGLViewPointer(this);
	
	SetViewRect(cRect);
	InitOpenGL();
	
	return nsObjPtr;
}

void CPPOpenGLView::Set3DViewPixelFormat(void)
{
	NSOpenGLPixelFormatAttribute attribs[24] =
	{
		// possibly look into:
		// NSOpenGLPFASampleBuffers
		// NSOpenGLPFASamples
		// NSOpenGLPFAAuxDepthStencil
		// NSOpenGLPFAMultisample
		
		NSOpenGLPFANoRecovery,
		NSOpenGLPFAWindow,
		NSOpenGLPFAAccelerated,
		NSOpenGLPFADoubleBuffer,
		NSOpenGLPFAColorSize, 24,
		NSOpenGLPFAAlphaSize, 8,
		NSOpenGLPFADepthSize, 24,
		NSOpenGLPFAStencilSize, 8,
		NSOpenGLPFAAccumSize, 0,
		0
	};
	NSOpenGLPixelFormat * format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
	[nsObjPtr setPixelFormat:format];
}

void CPPOpenGLView::MakeCurrentContext(void)
{
	[[nsObjPtr openGLContext] makeCurrentContext];
}

void CPPOpenGLView::SetCppNsOpenGLViewPointer(CPPOpenGLView *glvw)
{
	[nsObjPtr setCppNsOpenGLViewPointer:glvw];
}

void CPPOpenGLView::FlushBuffer(void)
{
	[[nsObjPtr openGLContext] flushBuffer];
}



/*****************************************************************************/
// CPPOpenGLView C++ class functions

CPPOpenGLView::CPPOpenGLView()
{
	// default view margins
	SetViewMargins(0,0,0,0);
}

CPPOpenGLView::~CPPOpenGLView()
{
}

void CPPOpenGLView::Draw(void)
{
}

bool CPPOpenGLView::Message(int msg, void *data)
{
	//printf("CPPOpenGLView::Message = %i \n",msg);
	switch(msg)
	{
		case cppResizeContent:
		{
			CGRect *cntRect = (CGRect*)data;
			if(cntRect) ResizeViewRect(*cntRect);
			break;
		}
	}
	
	return TRUE;
}

void CPPOpenGLView::SetPerspectiveProjection(CGRect &cntRect, GLdouble fov, GLdouble nclp, GLdouble fclp)
{
	GLfloat vpRatio = (GLfloat)cntRect.size.width/(GLfloat)cntRect.size.height;
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	//glViewport(cntRect.origin.x,cntRect.origin.y,cntRect.size.width,cntRect.size.height);
	glViewport(0,0,cntRect.size.width,cntRect.size.height);
	gluPerspective(fov,vpRatio,nclp,fclp);
}

void CPPOpenGLView::SetOrthoProjection(CGRect &cntRect)
{
	glViewport(0,0,cntRect.size.width,cntRect.size.height);
	
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	
	gluOrtho2D(0,cntRect.size.width,cntRect.size.height,0);
}

