/*
 *  CPPTextView.mm
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/17/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */

#include <AppKit/AppKit.h>
#include "CPPTextView.h"


/*****************************************************************************/
// Objective-C  Class CppNsTextView : NSTextView

@interface CppNsTextView : NSTextView
{ CPPTextView *cppTxtVw; }
- (void)setCppNsTextViewPointer:(CPPTextView*)txView;
- (CPPTextView*)getCppNsTextViewPointer;
@end


@implementation CppNsTextView
- (void)setCppNsTextViewPointer:(CPPTextView*)txView
{ cppTxtVw = txView; }

- (CPPTextView*)getCppNsTextViewPointer
{ return cppTxtVw; }

- (BOOL)isFlipped
{ return YES; }
@end



/*****************************************************************************/
// CPPTextView Objective-C call functions

id CPPTextView::InitWithFrame(CGRect cRect)
{
	NSRect contentRect = NSRectFromCGRect(cRect);
	nsObjPtr = [[[CppNsTextView alloc] initWithFrame:contentRect] autorelease];
	
	SetCppNsTextViewPointer(this);
	SetViewRect(cRect);
	
	return nsObjPtr;
}

void CPPTextView::SetCppNsTextViewPointer(CPPTextView *tvw)
{
	[nsObjPtr setCppNsTextViewPointer:tvw];
}

void CPPTextView::SetBackgroundColor(CGFloat r, CGFloat g, CGFloat b, CGFloat a)
{
	[nsObjPtr setBackgroundColor:GetNSColor(r,g,b,a)];
}

void CPPTextView::SetDrawsBackground(BOOL flag)
{
	[nsObjPtr setDrawsBackground:flag];
}

void CPPTextView::SetTextColor(CPPColor color)
{
	[nsObjPtr setTextColor:GetNSColor(color)];
	//[nsObjPtr setTextColor:[NSColor colorWithCalibratedRed:color.r green:color.g blue:color.b alpha:color.a]];
}

void CPPTextView::SetTextColor(CGFloat r, CGFloat g, CGFloat b, CGFloat a)
{
	[nsObjPtr setTextColor:GetNSColor(r,g,b,a)];
	//[nsObjPtr setTextColor:[NSColor colorWithCalibratedRed:r green:g blue:b alpha:a]];
}

void CPPTextView::SetTextContainerInset(CGFloat w, CGFloat h)
{
	[nsObjPtr setTextContainerInset:NSMakeSize(w,h)];
}

void CPPTextView::SetFont(int font, CGFloat size)
{
	[nsObjPtr setFont:GetNSFont(font,size)];
}

void CPPTextView::SetAlignment(int align)
{
	[nsObjPtr setAlignment:align];
}

void CPPTextView::LowerBaseline(int pts)
{
	for(int i=0; i<pts; i++)
		[nsObjPtr lowerBaseline:nil];
}

void CPPTextView::SetString(const char *str)
{
	[nsObjPtr setString:GetNSString(str)];
}

const char* CPPTextView::GetString(void)
{
	return [[nsObjPtr string] UTF8String];
}

void CPPTextView::MakeStaticText(void)
{
	SetAllowsUndo(FALSE);
	SetEditable(FALSE);
	SetSelectable(FALSE);
	SetFieldEditor(FALSE);
	SetRichText(FALSE);
	SetImportsGraphics(FALSE);
}

void CPPTextView::SetAllowsUndo(BOOL flag)
{
	[nsObjPtr setAllowsUndo:flag];
}

void CPPTextView::SetEditable(BOOL flag)
{
	[nsObjPtr setEditable:flag];
}

void CPPTextView::SetSelectable(BOOL flag)
{
	[nsObjPtr setSelectable:flag];
}

void CPPTextView::SetFieldEditor(BOOL flag)
{
	[nsObjPtr setFieldEditor:flag];
}

void CPPTextView::SetRichText(BOOL flag)
{
	[nsObjPtr setRichText:flag];
}

void CPPTextView::SetImportsGraphics(BOOL flag)
{
	[nsObjPtr setImportsGraphics:flag];
}

CGFloat CPPTextView::GetPointSize(void)
{
	return [[nsObjPtr font] pointSize];
}



/*****************************************************************************/
// CPPTextView C++ class functions

CPPTextView::CPPTextView()
{
	// default view margins
	SetViewMargins(0,0,0,0);
}

CPPTextView::~CPPTextView()
{
}

void CPPTextView::Draw(void)
{
}

bool CPPTextView::Message(int msg, void *data)
{
	return TRUE;
}


