/*
 *  CPPView.mm
 *  CPPAppKitTest
 *
 *  Created by Dan Libisch on 3/17/12.
 *  Copyright 2012 Libisch Graphic Design. All rights reserved.
 *
 */

#include <AppKit/AppKit.h>
#include "CPPView.h"
#include "CPPWindow.h"


/*****************************************************************************/
// Objective-C  Class CppNsView : NSView

@interface CppNsView : NSView
{ CPPView *cppView; }
- (void)setCppNsViewPointer:(CPPView*)view;
- (CPPView*)getCppNsViewPointer;
@end


@implementation CppNsView
- (void)setCppNsViewPointer:(CPPView*)view
{ cppView = view; }

- (CPPView*)getCppNsViewPointer
{ return cppView; }

- (BOOL)isFlipped
{ return YES; }
@end



/*****************************************************************************/
// CPPView Objective-C call functions

id CPPView::InitWithFrame(CGRect cRect)
{
	NSRect contentRect = NSRectFromCGRect(cRect);
	nsObjPtr = [[[CppNsView alloc] initWithFrame:contentRect] autorelease];

	SetCppNsViewPointer(this);
	SetViewRect(cRect);
	
	return nsObjPtr;
}

void CPPView::SetCppNsViewPointer(CPPView *vw)
{
	[nsObjPtr setCppNsViewPointer:vw];
}

void CPPView::SetAcceptsMouseMovedEvents(BOOL mm)
{
	[nsObjPtr setAcceptsMouseMovedEvents:mm];
}

bool CPPView::MouseInViewRect(CGPoint mPos)
{
	CGRect wndFrame = window->GetContentFrame();
	//CGRect wndFrame = GetWindowContentFrame();
	//CGRect wndFrame = GetViewContentFrame();
	CGPoint m = GetLocationInFrame(mPos, wndFrame);
	
	
	CGPoint min = vwRect.origin;
	CGPoint max = min;
	
	max.x += vwRect.size.width;
	max.y += vwRect.size.height;
	
	if(m.x > min.x && m.x < max.x && m.y > min.y && m.y < max.y) return TRUE;
	//if(mPos.x > min.x && mPos.x < max.x && mPos.y > min.y && mPos.y < max.y) return TRUE;
	else return FALSE;
}

void CPPView::AddSubview(CPPView *view)
{
	[nsObjPtr addSubview:view->GetNSObjectPointer()];
	view->SetParent(this);
	view->SetCPPWindow(GetCPPWindow());
}

void CPPView::SetFrame(CGRect rect)
{
	NSRect frm = NSRectFromCGRect(rect);
	[nsObjPtr setFrame:frm];
}

CGRect CPPView::GetFrame(void)
{
	NSRect frm = [nsObjPtr frame];
	return NSRectToCGRect(frm);
}

void CPPView::SetFrameSize(CGSize size)
{
	NSSize frmSize = NSSizeFromCGSize(size);
	[nsObjPtr setFrameSize:frmSize];
}

CGSize CPPView::GetFrameSize(void)
{
	NSRect frm = [nsObjPtr frame];
	return NSRectToCGRect(frm).size;
}

void CPPView::SetFrameOrigin(CGPoint origin)
{
	NSPoint frmOrigin = NSPointFromCGPoint(origin);
	[nsObjPtr setFrameOrigin:frmOrigin];
}

CGPoint CPPView::GetFrameOrigin(void)
{
	NSRect frm = [nsObjPtr frame];
	return NSRectToCGRect(frm).origin;
}

void CPPView::SetNeedsDisplay(BOOL flag)
{
	[nsObjPtr setNeedsDisplay:YES];
}

id CPPView::GetNSWindow(void)
{
	return [nsObjPtr window];
}

CGRect CPPView::GetSuperviewFrame(void)
{
	NSRect frmRect = [[nsObjPtr superview] frame];
	return NSRectToCGRect(frmRect);
}

CGRect CPPView::GetViewContentFrame(void)
{
	id wndPtr = GetNSWindow();
	NSRect frmRect = [wndPtr contentRectForFrameRect:[wndPtr frame]];
	//NSRect scrFrame = [[wndPtr screen] frame];
	NSRect scrFrame = [[[NSScreen screens] objectAtIndex:0] frame];
	
	frmRect.origin.x = frmRect.origin.x + vwRect.origin.x;
	frmRect.origin.y = (scrFrame.origin.y + scrFrame.size.height) - (frmRect.origin.y + frmRect.size.height - vwRect.origin.y);
	return NSRectToCGRect(frmRect);
}

CGRect CPPView::GetWindowFrame(void)
{
	return window->GetWindowFrame();
	//return window->GetContentFrame();
}

CGRect CPPView::GetWindowContentFrame(void)
{
	id wndPtr = GetNSWindow();
	NSRect frmRect = [wndPtr contentRectForFrameRect:[wndPtr frame]];
	//NSRect scrFrame = [[wndPtr screen] frame];
	NSRect scrFrame = [[[NSScreen screens] objectAtIndex:0] frame];
	
	frmRect.origin.y = (scrFrame.origin.y + scrFrame.size.height) - (frmRect.origin.y + frmRect.size.height);
	return NSRectToCGRect(frmRect);
}

CGRect CPPView::GetWindowContentRect(void)
{
	NSRect frmRect = [[[nsObjPtr window] contentView] frame];
	return NSRectToCGRect(frmRect);
}

void CPPView::Draw(void)
{
	for(int i=0; i < children->GetCount(); i++)
	{
		children->GetNode(i)->Draw();
	}
	[nsObjPtr display];
}




/*****************************************************************************/
// CPPView C++ class functions

CPPView::CPPView()
{
	window = NULL;
	vwRect = CGRectMake(0,0,0,0);
	
	// default view margins
	SetViewMargins(0,0,0,0);
	
}

CPPView::~CPPView()
{
}

bool CPPView::Message(int msg, void *data)
{
	//printf("CPPView::Message = %i \n",msg);
	switch(msg)
	{
		case cppResizeContent:
		{
			CGRect *cntRect = (CGRect*)data;
			if(cntRect)
			{
				ResizeViewRect(*cntRect);
				for(int i=0; i < children->GetCount(); i++)
				{
					CPPNode *ch = children->GetNode(i);
					if(ch)
					{
						ch->Message(cppResizeContent,cntRect);
						ch->Draw();
					}
				}
			}
			break;
		}
	}
	return TRUE;
}

void CPPView::SetCPPWindow(CPPWindow *wnd)
{
	window = wnd;
}

CPPWindow* CPPView::GetCPPWindow(void)
{
	return window;
}

void CPPView::SetViewRect(CGRect cRect)
{
	vwRect.origin.x = cRect.origin.x;
	vwRect.origin.y = cRect.origin.y;
	vwRect.size.width = cRect.size.width;
	vwRect.size.height = cRect.size.height;
	SetFrame(cRect);
}

CGRect CPPView::GetViewRect(void)
{
	return vwRect;
}

void CPPView::ResizeViewRect(CGRect wndRect)
{
	// resize origin x
	if(margins.left < 0)
		vwRect.origin.x = wndRect.size.width - vwRect.size.width - margins.right;
	else
		vwRect.origin.x = wndRect.origin.x + margins.left;
	
	// resize origin y
	/*if(margins.bottom < 0)
		vwRect.origin.y = wndRect.size.height - vwRect.size.height - margins.top;
	else
		vwRect.origin.y = wndRect.origin.y + margins.bottom;*/
	
	if(margins.top < 0)
		vwRect.origin.y = wndRect.size.height - vwRect.size.height - margins.bottom;
	else
		vwRect.origin.y = wndRect.origin.y + margins.top;
	
	// resize size width
	if(margins.right >= 0)
		vwRect.size.width = wndRect.size.width - margins.right - margins.left;
	//else if(margins.left > -1)
		//vwRect.size.width = wndRect.size.width - margins.right - margins.left;
	
	// resize size height
	/*if(margins.top < 0)
		vwRect.size.height = wndRect.size.height - margins.bottom;
	else if(margins.bottom > -1)
		vwRect.size.height = wndRect.size.height - margins.bottom - margins.top;*/

	if(margins.bottom >= 0)
		vwRect.size.height = wndRect.size.height - margins.bottom - margins.top;
	//else if(margins.top > -1)
		//vwRect.size.height = wndRect.size.height - margins.bottom - margins.top;
	
	// resize NSView frame
	SetFrameOrigin(vwRect.origin);
	SetFrameSize(vwRect.size);
}

void CPPView::SetViewOrigin(CGPoint origin)
{
	vwRect.origin = origin;
	SetFrameOrigin(origin);
}

CGPoint CPPView::GetViewOrigin(void)
{
	return vwRect.origin;
}

void CPPView::SetViewSize(CGSize size)
{
	vwRect.size = size;
	SetFrameSize(size);
}

CGSize CPPView::GetViewSize(void)
{
	return vwRect.size;
}

void CPPView::SetViewMargins(int l, int t, int r, int b)
{
	margins.left = l;
	margins.top = t;
	margins.right = r;
	margins.bottom = b;
}

void CPPView::SetViewLeftMargin(int l)
{
	margins.left = l;
}

void CPPView::SetViewTopMargin(int t)
{
	margins.top = t;
}

void CPPView::SetViewRightMargin(int r)
{
	margins.right = r;
}

void CPPView::SetViewBottomMargin(int b)
{
	margins.bottom = b;
}

ViewMargins CPPView::GetViewMargins(void)
{
	return margins;
}


