//
//  CPPWindow.mm
//  CPPAppKitTest
//
//  Created by Dan Libisch on 3/1/12.
//  Copyright 2012 Dan Libisch. All rights reserved.
//

#include <AppKit/AppKit.h>

#include "CPPWindow.h"
#include "CPPOpenGL3DViewPort.h"
#include "CPPApplication.h"


/*****************************************************************************/
// Objective-C  Class CppNsWindow : NSWindow

@interface CppNsWindow : NSWindow
{ CPPWindow *cppWnd; }
- (void)setCppNsWindowPointer:(CPPWindow*)wnd;
- (CPPWindow*)getCppNsWindowPointer;
@end


@implementation CppNsWindow
- (void)setCppNsWindowPointer:(CPPWindow*)wnd
{ cppWnd = wnd; }

- (CPPWindow*)getCppNsWindowPointer
{ return cppWnd; }

- (void)windowWillClose:(NSNotification *)notification
{ 
	if(CPPApp->Message(cppMainWindowClosesApplication))
		CPPApp->Message(cppTerminateApplication,self);
	else cppWnd->Message(cppWindowWillClose);
}

- (BOOL)windowShouldClose:(NSNotification *)notification
{ return cppWnd->Message(cppWindowShouldClose); }

- (void)windowDidResize:(NSNotification *)notification
{ cppWnd->Message(cppWindowDidResize); }

- (void)windowDidMove:(NSNotification *)notification
{ cppWnd->Message(cppWindowDidMove); }

// Only used to stop alert sound when keys are pressed
- (void)keyDown:(NSEvent *)theEvent { }
@end




/*****************************************************************************/
// CPPWindow Objective-C call functions

void CPPWindow::InitWithContentRect(CGRect cRect, unsigned int style)
{
	NSRect contentR = NSRectFromCGRect(cRect);
	nsObjPtr = [[[CppNsWindow alloc] initWithContentRect:contentR styleMask:style backing:NSBackingStoreBuffered defer:NO] autorelease];
	
 	SetCppNsWindowPointer(this);
	UpdateWindowFrame();
	CreateContentView(cRect);
	SetWindowIsOpen(TRUE);
	
	NSRect screenR = [[nsObjPtr screen] visibleFrame];
	contentR.origin = NSMakePoint(screenR.origin.x, screenR.origin.y + screenR.size.height - contentR.size.height);
	[nsObjPtr setFrame:[nsObjPtr frameRectForContentRect:contentR] display:YES animate:YES];
}

void CPPWindow::SetCppNsWindowPointer(CPPWindow *wnd)
{
	[nsObjPtr setCppNsWindowPointer:wnd];
}

void CPPWindow::SetContentView(id view)
{
	[nsObjPtr setContentView:view];
}

void CPPWindow::AddViewToWindowContent(CPPView *view)
{
	[[nsObjPtr contentView] addSubview:view->GetNSObjectPointer()];
	view->SetParent(cntView);
	view->SetCPPWindow(this);
}

void CPPWindow::SetMinSize(CGFloat w, CGFloat h)
{
	[nsObjPtr setContentMinSize:NSMakeSize(w,h)];
}

void CPPWindow::CloseWindow(void)
{
	[nsObjPtr performClose: nsObjPtr];
}

CGRect CPPWindow::GetContentRect(void)
{
	return NSRectToCGRect( [[nsObjPtr contentView] frame] );
}

void CPPWindow::SetDelegate(id del)
{
	[nsObjPtr setDelegate:del];
}

void CPPWindow::MakeFirstResponder(id rspnd)
{
	[nsObjPtr makeFirstResponder:rspnd];
}

void CPPWindow::SetAcceptsMouseMovedEvents(BOOL mm)
{
	[nsObjPtr setAcceptsMouseMovedEvents:mm];
}

void CPPWindow::SetTitle(id name)
{
	[nsObjPtr setTitle:name];
}

void CPPWindow::MakeKeyAndOrderFront(void)
{
    [nsObjPtr makeKeyAndOrderFront:nil];
	SetWindowIsOpen(TRUE);
}

void CPPWindow::SetContentSize(CGSize size)
{
	cntView->GetViewRect().size = size;
	
	NSSize cntSize = NSSizeFromCGSize(size);
	[nsObjPtr setContentSize:cntSize];
}

void CPPWindow::SetBackgroundColor(CPPColor color)
{
	return [nsObjPtr setBackgroundColor:GetNSColor(color)];
}

void CPPWindow::SetBackgroundColor(CGFloat r, CGFloat g, CGFloat b, CGFloat a)
{
	return [nsObjPtr setBackgroundColor:GetNSColor(r,g,b,a)];
}

void CPPWindow::SetFrameOrigin(CGPoint origin)
{
	NSPoint frmOrigin = NSPointFromCGPoint(origin);
	//NSRect screenFrame = [[nsObjPtr screen] frame];
	NSRect screenFrame = [[[NSScreen screens] objectAtIndex:0] frame];
	NSSize wndSize = [nsObjPtr frame].size;
	
	frmOrigin.y = screenFrame.size.height - origin.y - wndSize.height;
	[nsObjPtr setFrameOrigin:frmOrigin];
	UpdateWindowFrame();
}

void CPPWindow::SetFrameTopLeftPoint(CGPoint topLeft)
{
	NSPoint frmTopLeft = NSPointFromCGPoint(topLeft);
	//NSRect screenFrame = [[nsObjPtr screen] frame];
	NSRect screenFrame = [[[NSScreen screens] objectAtIndex:0] frame];

	frmTopLeft.y = screenFrame.size.height - topLeft.y;
	//printf("frmTopLeft: x = %f, y = %f\n", frmTopLeft.x, frmTopLeft.y);
	[nsObjPtr setFrameTopLeftPoint:frmTopLeft];
	UpdateWindowFrame();
}

void CPPWindow::UpdateWindowFrame(void)
{
	//NSRect screenFrame = [[nsObjPtr screen] frame];
	NSRect screenFrame = [[[NSScreen screens] objectAtIndex:0] frame];
	
	NSRect windowFrame = [nsObjPtr frame];
	wndFrame = NSRectToCGRect(windowFrame);
	wndFrame.origin.y = screenFrame.size.height - windowFrame.origin.y - windowFrame.size.height;
	
	NSRect contentFrame = [nsObjPtr contentRectForFrameRect:[nsObjPtr frame]];
	cntFrame = NSRectToCGRect(contentFrame);
	cntFrame.origin.y = screenFrame.size.height - contentFrame.origin.y - contentFrame.size.height;
}


/*****************************************************************************/
// CPPOpenGLView C++ class functions

CPPWindow::CPPWindow()
{
	wndFrame = CGRectMake(0,0,0,0);
	cntFrame = CGRectMake(0,0,0,0);
	
	cntView = NULL;
	wndIsOpen = FALSE;
}

CPPWindow::~CPPWindow()
{
}

bool CPPWindow::CreateContentView(CGRect cRect)
{
	cntView = new CPPView(); if(!cntView) return FALSE;
	cntView->SetParent(this);
	cntView->SetCPPWindow(this);
	
	id contentView = cntView->InitWithFrame(cRect);
	SetContentView(contentView);
	SetContentRect(cRect);
}

CPPView* CPPWindow::GetContentView(void)
{
	return cntView;
}

void CPPWindow::SetWindowIsOpen(bool flag)
{
	wndIsOpen = flag;
}

bool CPPWindow::WindowIsOpen(void)
{
	return wndIsOpen;
}

void CPPWindow::SetContentRect(CGRect cRect)
{
	CGRect cntRect = cRect;
	cntRect.origin.y = cntFrame.size.height - cntRect.size.height;
	cntView->SetViewRect(cntRect);
}

CGRect CPPWindow::GetWindowFrame(void)
{
	return wndFrame;
}

CGRect CPPWindow::GetContentFrame(void)
{
	return cntFrame;
}

void CPPWindow::Draw(void)
{
	//printf("CPPWindow::Draw \n");
	for(int i=0; i < children->GetCount(); i++)
	{
		children->GetNode(i)->Draw();
	}
}

bool CPPWindow::WindowShouldClose(void)
{
	//if window should not close return FALSE; else
	return TRUE;
}

bool CPPWindow::WindowWillClose(void)
{
	SetWindowIsOpen(FALSE);
	Remove();
	return TRUE;
}

bool CPPWindow::WindowDidResize(void)
{
	//printf("CPPWindow::WindowDidResize \n");
	CGRect cRect = GetContentRect();
	UpdateWindowFrame();
	
	for(int i=0; i < children->GetCount(); i++)
	{
		CPPNode *ch = children->GetNode(i);
		if(ch)
		{
			ch->Message(cppResizeContent,&cRect);
			ch->Draw();
		}
	}
	
	return TRUE;
}

bool CPPWindow::WindowDidMove(void)
{
	UpdateWindowFrame();
	return TRUE;
}

bool CPPWindow::Message(int msg, void *data)
{
	switch(msg)
	{
		case cppWindowShouldClose:
		{
			//printf("CPPWindow::Message = cppWindowShouldClose \n");
			return WindowShouldClose();
		}
		case cppWindowWillClose:
		{
			//printf("CPPWindow::Message = cppWindowWillClose \n");
			return WindowWillClose();
		}
		case cppWindowDidResize:
		{
			//printf("CPPWindow::Message = cppWindowDidResize \n");
			return WindowDidResize();
		}
		case cppWindowDidMove:
		{
			//printf("CPPWindow::Message = cppWindowDidMove \n");
			return WindowDidMove();
		}
	}
	
	return TRUE;
}

